"""
Test breakpoint commands for a breakpoint ID with multiple locations.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BreakpointLocationsTestCase(TestBase):
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24528")
    def test_enable(self):
        """Test breakpoint enable/disable for a breakpoint ID with multiple locations."""
        self.build()
        self.breakpoint_locations_test()

    def test_shadowed_cond_options(self):
        """Test that options set on the breakpoint and location behave correctly."""
        self.build()
        self.shadowed_bkpt_cond_test()

    def test_shadowed_command_options(self):
        """Test that options set on the breakpoint and location behave correctly."""
        self.build()
        self.shadowed_bkpt_command_test()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.c", "// Set break point at this line.")

    def set_breakpoint(self):
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, "Target %s is not valid" % (exe))

        # This should create a breakpoint with 3 locations.

        bkpt = target.BreakpointCreateByLocation("main.c", self.line)

        # The breakpoint list should show 3 locations.
        self.assertEqual(bkpt.GetNumLocations(), 3, "Wrong number of locations")

        self.expect(
            "breakpoint list -f",
            "Breakpoint locations shown correctly",
            ordered=False,
            substrs=[
                f"1: file = 'main.c', line = {self.line}, exact_match = 0, locations = 3"
            ],
            patterns=[
                "where = a.out`func_inlined .+?unresolved, hit count = 0",
                r"where = a.out`main .+?\[inlined\].+?unresolved, hit count = 0",
            ],
        )

        return bkpt

    def shadowed_bkpt_cond_test(self):
        """Test that options set on the breakpoint and location behave correctly."""
        # Breakpoint option propagation from bkpt to loc used to be done the first time
        # a breakpoint location option was specifically set.  After that the other options
        # on that location would stop tracking the breakpoint.  That got fixed, and this test
        # makes sure only the option touched is affected.

        bkpt = self.set_breakpoint()
        bkpt_cond = "1 == 0"
        bkpt.SetCondition(bkpt_cond)
        self.assertEqual(bkpt.GetCondition(), bkpt_cond, "Successfully set condition")
        self.assertEqual(
            bkpt.location[0].GetCondition(),
            bkpt.GetCondition(),
            "Conditions are the same",
        )

        # Now set a condition on the locations, make sure that this doesn't effect the bkpt:
        bkpt_loc_1_cond = "1 == 1"
        bkpt.location[0].SetCondition(bkpt_loc_1_cond)
        self.assertEqual(
            bkpt.location[0].GetCondition(),
            bkpt_loc_1_cond,
            "Successfully changed location condition",
        )
        self.assertNotEqual(
            bkpt.GetCondition(),
            bkpt_loc_1_cond,
            "Changed location changed Breakpoint condition",
        )
        self.assertEqual(
            bkpt.location[1].GetCondition(),
            bkpt_cond,
            "Changed another location's condition",
        )

        # Now make sure that setting one options doesn't fix the value of another:
        bkpt.SetIgnoreCount(10)
        self.assertEqual(bkpt.GetIgnoreCount(), 10, "Set the ignore count successfully")
        self.assertEqual(
            bkpt.location[0].GetIgnoreCount(),
            10,
            "Location doesn't track top-level bkpt.",
        )

        # Now make sure resetting the condition to "" resets the tracking:
        bkpt.location[0].SetCondition("")
        bkpt_new_cond = "1 == 3"
        bkpt.SetCondition(bkpt_new_cond)
        self.assertEqual(
            bkpt.location[0].GetCondition(),
            bkpt_new_cond,
            "Didn't go back to tracking condition",
        )

        # Test that set/get accessor methods on BreakpointLocation behave correctly.
        bkpt_loc = bkpt.GetLocationAtIndex(0)

        value = "MyQueue"
        bkpt_loc.SetQueueName(value)
        self.assertEqual(
            bkpt_loc.GetQueueName(), value, "Successfully set/get bp location QueueName"
        )

        value = 5
        bkpt_loc.SetThreadID(value)
        self.assertEqual(
            bkpt_loc.GetThreadID(), value, "Successfully set/get bp location ThreadID"
        )

        value = "1 == 0"
        bkpt_loc.SetCondition(value)
        self.assertEqual(
            bkpt_loc.GetCondition(), value, "Successfully set/get bp location Condition"
        )

        value = 6
        bkpt_loc.SetThreadIndex(value)
        self.assertEqual(
            bkpt_loc.GetThreadIndex(),
            value,
            "Successfully set/get bp location ThreadIndex",
        )

        value = "MyThread"
        bkpt_loc.SetThreadName(value)
        self.assertEqual(
            bkpt_loc.GetThreadName(),
            value,
            "Successfully set/get bp location ThreadName",
        )

        value = 5
        bkpt_loc.SetIgnoreCount(value)
        self.assertEqual(
            bkpt_loc.GetIgnoreCount(),
            value,
            "Successfully set/get bp location IgnoreCount",
        )

        for value in [True, False]:
            bkpt_loc.SetAutoContinue(value)
            self.assertEqual(
                bkpt_loc.GetAutoContinue(),
                value,
                "Successfully set/get bp location AutoContinue",
            )

        for value in [True, False]:
            bkpt_loc.SetEnabled(value)
            self.assertEqual(
                bkpt_loc.IsEnabled(),
                value,
                "Successfully set/get bp location SetEnabled",
            )

        # test set/get CommandLineCommands
        set_cmds = lldb.SBStringList()
        set_cmds.AppendString("frame var")
        set_cmds.AppendString("bt")
        bkpt_loc.SetCommandLineCommands(set_cmds)

        get_cmds = lldb.SBStringList()
        bkpt_loc.GetCommandLineCommands(get_cmds)
        self.assertEqual(
            set_cmds.GetSize(), get_cmds.GetSize(), "Size of command line commands"
        )
        for idx, _ in enumerate(set_cmds):
            self.assertEqual(
                set_cmds.GetStringAtIndex(idx),
                get_cmds.GetStringAtIndex(idx),
                "Command %d" % (idx),
            )

    def shadowed_bkpt_command_test(self):
        """Test that options set on the breakpoint and location behave correctly."""
        # Breakpoint option propagation from bkpt to loc used to be done the first time
        # a breakpoint location option was specifically set.  After that the other options
        # on that location would stop tracking the breakpoint.  That got fixed, and this test
        # makes sure only the option touched is affected.

        bkpt = self.set_breakpoint()
        commands = ["AAAAAA", "BBBBBB", "CCCCCC"]
        str_list = lldb.SBStringList()
        str_list.AppendList(commands, len(commands))

        bkpt.SetCommandLineCommands(str_list)
        cmd_list = lldb.SBStringList()
        bkpt.GetCommandLineCommands(cmd_list)
        list_size = str_list.GetSize()
        self.assertEqual(
            cmd_list.GetSize(), list_size, "Added the right number of commands"
        )
        for i in range(0, list_size):
            self.assertEqual(
                str_list.GetStringAtIndex(i),
                cmd_list.GetStringAtIndex(i),
                "Mismatched commands.",
            )

        commands = ["DDDDDD", "EEEEEE", "FFFFFF", "GGGGGG"]
        loc_list = lldb.SBStringList()
        loc_list.AppendList(commands, len(commands))
        bkpt.location[1].SetCommandLineCommands(loc_list)
        loc_cmd_list = lldb.SBStringList()
        bkpt.location[1].GetCommandLineCommands(loc_cmd_list)

        loc_list_size = loc_list.GetSize()

        # Check that the location has the right commands:
        self.assertEqual(
            loc_cmd_list.GetSize(),
            loc_list_size,
            "Added the right number of commands to location",
        )
        for i in range(0, loc_list_size):
            self.assertEqual(
                loc_list.GetStringAtIndex(i),
                loc_cmd_list.GetStringAtIndex(i),
                "Mismatched commands.",
            )

        # Check that we didn't mess up the breakpoint level commands:
        self.assertEqual(
            cmd_list.GetSize(), list_size, "Added the right number of commands"
        )
        for i in range(0, list_size):
            self.assertEqual(
                str_list.GetStringAtIndex(i),
                cmd_list.GetStringAtIndex(i),
                "Mismatched commands.",
            )

        # And check we didn't mess up another location:
        untouched_loc_cmds = lldb.SBStringList()
        bkpt.location[0].GetCommandLineCommands(untouched_loc_cmds)
        self.assertEqual(untouched_loc_cmds.GetSize(), 0, "Changed the wrong location")

    def breakpoint_locations_test(self):
        """Test breakpoint enable/disable for a breakpoint ID with multiple locations."""
        self.set_breakpoint()

        # The 'breakpoint disable 3.*' command should fail gracefully.
        self.expect(
            "breakpoint disable 3.*",
            "Disabling an invalid breakpoint should fail gracefully",
            error=True,
            startstr="error: '3' is not a valid breakpoint ID.",
        )

        # The 'breakpoint disable 1.*' command should disable all 3 locations.
        self.expect(
            "breakpoint disable 1.*",
            "All 3 breakpoint locatons disabled correctly",
            startstr="3 breakpoints disabled.",
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should not stopped on any breakpoint at all.
        self.expect(
            "process status",
            "No stopping on any disabled breakpoint",
            patterns=["^Process [0-9]+ exited with status = 0"],
        )

        # The 'breakpoint enable 1.*' command should enable all 3 breakpoints.
        self.expect(
            "breakpoint enable 1.*",
            "All 3 breakpoint locatons enabled correctly",
            startstr="3 breakpoints enabled.",
        )

        # The 'breakpoint enable 1.' command should not crash.
        self.expect(
            "breakpoint enable 1.",
            startstr="0 breakpoints enabled.",
        )

        # The 'breakpoint disable 1.1' command should disable 1 location.
        self.expect(
            "breakpoint disable 1.1",
            "1 breakpoint locatons disabled correctly",
            startstr="1 breakpoints disabled.",
        )

        # Run the program again.  We should stop on the two breakpoint
        # locations.
        self.runCmd("run", RUN_SUCCEEDED)

        # Stopped once.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        # Continue the program, there should be another stop.
        self.runCmd("process continue")

        # Stopped again.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        # At this point, 1.1 has a hit count of 0 and the other a hit count of
        # 1".
        lldbutil.check_breakpoint(
            self,
            bpno=1,
            expected_locations=3,
            expected_resolved_count=2,
            expected_hit_count=2,
        )
        lldbutil.check_breakpoint(
            self,
            bpno=1,
            location_id=1,
            expected_location_resolved=False,
            expected_location_hit_count=0,
        )
        lldbutil.check_breakpoint(
            self,
            bpno=1,
            location_id=2,
            expected_location_resolved=True,
            expected_location_hit_count=1,
        )
        lldbutil.check_breakpoint(
            self,
            bpno=1,
            location_id=3,
            expected_location_resolved=True,
            expected_location_hit_count=1,
        )



# -*- coding: utf-8 -*-
#
# LLDB documentation build configuration file, created by
# sphinx-quickstart on Sun Dec  9 20:01:55 2012.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.
import sys, os, re, shutil
from datetime import date

# Add path for llvm_slug module.
sys.path.insert(0, os.path.abspath(os.path.join("..", "..", "llvm", "docs")))

building_man_page = tags.has("builder-man")

# For the website we need to setup the path to the generated LLDB module that
# we can generate documentation for its API.
if not building_man_page:
    # If extensions (or modules to document with autodoc) are in another directory,
    # add these directories to sys.path here. If the directory is relative to the
    # documentation root, use os.path.abspath to make it absolute, like shown here.

    # Add the current directory that contains the mock _lldb native module which
    # is imported by the `lldb` module.
    sys.path.insert(0, os.path.abspath("."))
    # Add the build directory that contains the `lldb` module. LLDB_SWIG_MODULE is
    # set by CMake.
    sys.path.insert(0, os.getenv("LLDB_SWIG_MODULE"))

# Put the generated Python API documentation in the 'python_api' folder. This
# also defines the URL these files will have in the generated website.
automodapi_toctreedirnm = "python_api"

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.todo",  # Support for todo items.
    "sphinx.ext.mathjax",  # Render math via JavaScript.
    "sphinx.ext.intersphinx",  # Link to other projects’ documentation.
    "sphinx.ext.napoleon",  # Support for NumPy and Google style docstrings.
]

# When building man pages, we do not use the markdown pages,
# So, we can continue without the myst_parser dependencies.
# Doing so reduces dependencies of some packaged llvm distributions.
try:
    import myst_parser

    extensions.append("myst_parser")
except ImportError:
    if not tags.has("builder-man"):
        raise

# Automatic anchors for markdown titles
myst_heading_anchors = 6
myst_heading_slug_func = "llvm_slug.make_slug"

autodoc_default_options = {"special-members": True}

# The suffix of source filenames.
source_suffix = {
    ".rst": "restructuredtext",
}

# Unless we only generate the basic manpage we need the plugin for generating
# the Python API documentation.
if not building_man_page:
    try:
        import sphinx_automodapi.automodapi
    except ModuleNotFoundError:
        print(
            f"install sphinx_automodapi with {sys.executable} -m pip install sphinx_automodapi"
        )
    extensions.append("sphinx_automodapi.automodapi")

    try:
        import furo
    except ModuleNotFoundError:
        print(f"install sphinx furo theme with {sys.executable} -m pip install furo")
    # The theme to use for HTML and HTML Help pages.  See the documentation for
    # a list of builtin themes.
    html_theme = "furo"

    # Since man pages do not use markdown, we do not need to register a markdown
    # parser.
    source_suffix[".md"] = "markdown"

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "LLDB"
copyright = "2007-%d, The LLDB Team" % date.today().year

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents. These are currently set to zero because we don't use them.
# Should somebody consider in the future to change them, they need to be updated
# everytime a new release comes out.
#
# The short version.
# version = '0'
# The full version, including alpha/beta/rc tags.
# release = '0'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ["_build", "analyzer"]
# Ignore the generated Python documentation that is only used on the website.
# Without this we will get a lot of warnings about doc pages that aren't
# included by any doctree (as the manpage ignores those pages but they are
# potentially still around from a previous website generation).
if building_man_page:
    exclude_patterns.append(automodapi_toctreedirnm)
# Use the recommended 'any' rule so that referencing SB API classes is possible
# by just writing `SBData`.
default_role = "any"

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "friendly"

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "furo"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    "source_repository": "https://github.com/llvm/llvm-project",
    "source_branch": "main",
    "source_directory": "lldb/docs/",
}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "🐛 LLDB"

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]

html_extra_path = [".htaccess"]

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = "%b %d, %Y"

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = "LLDBdoc"

# If true, the reST sources are included in the HTML build as
# _sources/name. The default is True.
html_copy_source = False

# -- Options for LaTeX output --------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    ("index", "LLDB.tex", "LLDB Documentation", "The LLDB Team", "manual"),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ("man/lldb", "lldb", "LLDB Documentation", ["LLVM project"], 1),
    ("man/lldb-server", "lldb-server", "LLDB Documentation", ["LLVM project"], 1),
]

# If true, show URL addresses after external links.
# man_show_urls = False

# -- Options for Texinfo output ------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        "index",
        "LLDB",
        "LLDB Documentation",
        "The LLDB Team",
        "LLDB",
        "One line description of project.",
        "Miscellaneous",
    ),
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

empty_attr_summary = re.compile(
    r"\.\. rubric:: Attributes Summary\s*\.\. autosummary::\s*\.\. rubric::"
)
empty_attr_documentation = re.compile(
    r"\.\. rubric:: Attributes Documentation\s*\.\. rubric::"
)


def preprocess_source(app, docname, source):
    """Preprocesses source files generated by automodapi."""
    # Don't cleanup anything beside automodapi-generated sources.
    if not automodapi_toctreedirnm in docname:
        return
    processed = source[0]

    # Don't show the list of inheritance info as there is no inheritance in the
    # SBI API. This avoids all the repeated text on all doc pages that a
    # class inherits from 'object'.

    processed = processed.replace(":show-inheritance:", "")
    # Remove the SWIG generated 'thisown' attribute. It just bloats the generated
    # documentation and users shouldn't fiddle with the value anyway.
    processed = re.sub(r"~SB[a-zA-Z]+\.thisown", "", processed)
    processed = processed.replace("  .. autoattribute:: thisown", "")

    # After removing 'thisown', many objects don't have any attributes left.
    # Remove all now empty attribute summary/documentation sections with
    # some rather ugly regex.
    processed = empty_attr_summary.sub(".. rubric::", processed)
    processed = empty_attr_documentation.sub(".. rubric::", processed)

    # Replace the original source with the processed one (source is a single
    # element list).
    source[0] = processed


def cleanup_source(app, exception):
    """Remove files generated by automodapi in the source tree."""
    if hasattr(app.config, "automodapi_toctreedirnm"):
        api_source_dir = os.path.join(app.srcdir, app.config.automodapi_toctreedirnm)
        shutil.rmtree(api_source_dir, ignore_errors=True)


def setup(app):
    app.connect("source-read", preprocess_source)
    app.connect("build-finished", cleanup_source)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestGDBRemoteLoad(GDBRemoteTestBase):
    def test_module_load_address(self):
        """Test that setting the load address of a module uses virtual addresses"""
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        module = target.GetModuleAtIndex(0)
        self.assertTrue(module.IsValid())
        self.assertTrue(target.SetModuleLoadAddress(module, 0).Success())
        address = target.ResolveLoadAddress(0x2001)
        self.assertTrue(address.IsValid())
        self.assertEqual(".data", address.GetSection().GetName())

    def test_ram_load(self):
        """Test loading an object file to a target's ram"""
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        self.dbg.HandleCommand("target modules load -l -s0")
        self.assertPacketLogReceived(["M1000,4:c3c3c3c3", "M1004,2:3232"])

    @skipIfXmlSupportMissing
    def test_flash_load(self):
        """Test loading an object file to a target's flash memory"""

        class Responder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;qXfer:memory-map:read+"

            def qXferRead(self, obj, annex, offset, length):
                if obj == "memory-map":
                    return (
                        self.MEMORY_MAP[offset : offset + length],
                        offset + length < len(self.MEMORY_MAP),
                    )
                return None, False

            def other(self, packet):
                if packet[0:11] == "vFlashErase":
                    return "OK"
                if packet[0:11] == "vFlashWrite":
                    return "OK"
                if packet == "vFlashDone":
                    return "OK"
                return ""

            MEMORY_MAP = """<?xml version="1.0"?>
<memory-map>
  <memory type="ram" start="0x0" length="0x1000"/>
  <memory type="flash" start="0x1000" length="0x1000">
    <property name="blocksize">0x100</property>
  </memory>
  <memory type="ram" start="0x2000" length="0x1D400"/>
</memory-map>
"""

        self.server.responder = Responder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        self.dbg.HandleCommand("target modules load -l -s0")
        self.assertPacketLogReceived(
            [
                "vFlashErase:1000,100",
                "vFlashWrite:1000:\xc3\xc3\xc3\xc3",
                "vFlashWrite:1004:\x32\x32",
                "vFlashDone",
            ]
        )



# RUN: %PYTHON %s 2>&1

import os

from mlir._mlir_libs import get_include_dirs, get_lib_dirs


header_file = os.path.join(get_include_dirs()[0], "mlir-c", "IR.h")
assert os.path.isfile(header_file), f"Header does not exist: {header_file}"

# Since actual library names are platform specific, just scan the directory
# for a filename that contains the library name.
expected_lib_name = "MLIRPythonCAPI"
all_libs = os.listdir(get_lib_dirs()[0])
found_lib = False
for file_name in all_libs:
    if expected_lib_name in file_name:
        found_lib = True
assert found_lib, f"Did not find '{expected_lib_name}' lib in {all_libs}"



import lldb
import time
import unittest
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbreverse import ReverseTestBase
from lldbsuite.test import lldbutil


class TestReverseContinueBreakpoints(ReverseTestBase):
    @skipIfRemote
    def test_reverse_continue(self):
        self.reverse_continue_internal(async_mode=False)

    @skipIfRemote
    def test_reverse_continue_async(self):
        self.reverse_continue_internal(async_mode=True)

    def reverse_continue_internal(self, async_mode):
        target, process, initial_threads = self.setup_recording(async_mode)

        # Reverse-continue. We'll stop at the point where we started recording.
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.assertSuccess(status)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        self.expect(
            "thread list",
            STOPPED_DUE_TO_HISTORY_BOUNDARY,
            substrs=["stopped", "stop reason = history boundary"],
        )

        # Continue forward normally until the target exits.
        status = process.ContinueInDirection(lldb.eRunForward)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateExited]
        )
        self.assertSuccess(status)
        self.assertState(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)

    @skipIfRemote
    def test_reverse_continue_breakpoint(self):
        self.reverse_continue_breakpoint_internal(async_mode=False)

    @skipIfRemote
    def test_reverse_continue_breakpoint_async(self):
        self.reverse_continue_breakpoint_internal(async_mode=True)

    def reverse_continue_breakpoint_internal(self, async_mode):
        target, process, initial_threads = self.setup_recording(async_mode)

        # Reverse-continue to the function "trigger_breakpoint".
        trigger_bkpt = target.BreakpointCreateByName("trigger_breakpoint", None)
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.assertSuccess(status)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        threads_now = lldbutil.get_threads_stopped_at_breakpoint(process, trigger_bkpt)
        self.assertEqual(threads_now, initial_threads)

    @skipIfRemote
    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/138084",
    )
    def test_reverse_continue_skip_breakpoint(self):
        self.reverse_continue_skip_breakpoint_internal(async_mode=False)

    @skipIfRemote
    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/138084",
    )
    def test_reverse_continue_skip_breakpoint_async(self):
        self.reverse_continue_skip_breakpoint_internal(async_mode=True)

    def reverse_continue_skip_breakpoint_internal(self, async_mode):
        target, process, initial_threads = self.setup_recording(async_mode)

        # Reverse-continue over a breakpoint at "trigger_breakpoint" whose
        # condition is false (via function call).
        # This tests that we continue in the correct direction after hitting
        # the breakpoint.
        trigger_bkpt = target.BreakpointCreateByName("trigger_breakpoint", None)
        trigger_bkpt.SetCondition("false_condition()")
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        self.assertSuccess(status)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_HISTORY_BOUNDARY,
            substrs=["stopped", "stop reason = history boundary"],
        )

    @skipIfRemote
    def test_continue_preserves_direction(self):
        self.continue_preserves_direction_internal(async_mode=False)

    @skipIfRemote
    def test_continue_preserves_direction_asyhc(self):
        self.continue_preserves_direction_internal(async_mode=True)

    def continue_preserves_direction_internal(self, async_mode):
        target, process, initial_threads = self.setup_recording(async_mode)

        # Reverse-continue to the function "trigger_breakpoint".
        trigger_bkpt = target.BreakpointCreateByName("trigger_breakpoint", None)
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.assertSuccess(status)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        # This should continue in reverse.
        status = process.Continue()
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        self.assertSuccess(status)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_HISTORY_BOUNDARY,
            substrs=["stopped", "stop reason = history boundary"],
        )

    def setup_recording(self, async_mode):
        """
        Record execution of code between "start_recording" and "stop_recording" breakpoints.

        Returns with the target stopped at "stop_recording", with recording disabled,
        ready to reverse-execute.
        """
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        process = self.connect(target)

        # Record execution from the start of the function "start_recording"
        # to the start of the function "stop_recording". We want to keep the
        # interval that we record as small as possible to minimize the run-time
        # of our single-stepping recorder.
        start_recording_bkpt = target.BreakpointCreateByName("start_recording", None)
        self.assertTrue(start_recording_bkpt.GetNumLocations() > 0)
        initial_threads = lldbutil.continue_to_breakpoint(process, start_recording_bkpt)
        self.assertEqual(len(initial_threads), 1)
        target.BreakpointDelete(start_recording_bkpt.GetID())
        self.start_recording()
        stop_recording_bkpt = target.BreakpointCreateByName("stop_recording", None)
        self.assertTrue(stop_recording_bkpt.GetNumLocations() > 0)
        lldbutil.continue_to_breakpoint(process, stop_recording_bkpt)
        target.BreakpointDelete(stop_recording_bkpt.GetID())
        self.stop_recording()

        self.dbg.SetAsync(async_mode)
        self.expect_async_state_changes(async_mode, process, [lldb.eStateStopped])

        return target, process, initial_threads

    def expect_async_state_changes(self, async_mode, process, states):
        if not async_mode:
            return
        listener = self.dbg.GetListener()
        lldbutil.expect_state_changes(self, listener, process, states)



# RUN: %PYTHON %s
"""
This script generates multi-threaded tests to check free-threading mode using CPython compiled with TSAN.
Tests can be run using pytest:
```bash
python3.13t -mpytest -vvv multithreaded_tests.py
```

IMPORTANT. Running tests are not checking the correctness, but just the execution of the tests in multi-threaded context
and passing if no warnings reported by TSAN and failing otherwise.


Details on the generated tests and execution:
1) Multi-threaded execution: all generated tests are executed independently by
a pool of threads, running each test multiple times, see @multi_threaded for details

2) Tests generation: we use existing tests: test/python/ir/*.py,
test/python/dialects/*.py, etc to generate multi-threaded tests.
In details, we perform the following:
a) we define a list of source tests to be used to generate multi-threaded tests, see `TEST_MODULES`.
b) we define `TestAllMultiThreaded` class and add existing tests to the class. See `add_existing_tests` method.
c) for each test file, we copy and modify it: test/python/ir/affine_expr.py -> /tmp/ir/affine_expr.py.
In order to import the test file as python module, we remove all executing functions, like
`@run` or `run(testMethod)`. See `copy_and_update` and `add_existing_tests` methods for details.


Observed warnings reported by TSAN.

CPython and free-threading known data-races:
1) ctypes related races: https://github.com/python/cpython/issues/127945
2) LLVM related data-races, llvm::raw_ostream is not thread-safe
- mlir pass manager
- dialects/transform_interpreter.py
- ir/diagnostic_handler.py
- ir/module.py
3) Dialect gpu module-to-binary method is unsafe
"""
import concurrent.futures
import gc
import importlib.util
import os
import sys
import threading
import tempfile
import unittest

from contextlib import contextmanager
from functools import partial
from pathlib import Path
from typing import Optional, List

import mlir.dialects.arith as arith
from mlir.dialects import transform
from mlir.ir import Context, Location, Module, IntegerType, InsertionPoint


def import_from_path(module_name: str, file_path: Path):
    spec = importlib.util.spec_from_file_location(module_name, file_path)
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    return module


def copy_and_update(src_filepath: Path, dst_filepath: Path):
    # We should remove all calls like `run(testMethod)`
    with open(src_filepath, "r") as reader, open(dst_filepath, "w") as writer:
        while True:
            src_line = reader.readline()
            if len(src_line) == 0:
                break
            skip_lines = [
                "run(",
                "@run",
                "@constructAndPrintInModule",
                "run_apply_patterns(",
                "@run_apply_patterns",
                "@test_in_context",
                "@construct_and_print_in_module",
            ]
            if any(src_line.startswith(line) for line in skip_lines):
                continue
            writer.write(src_line)


# Helper run functions
def run(f):
    f()


def run_with_context_and_location(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        f()
    return f


def run_with_insertion_point(f):
    print("\nTEST:", f.__name__)
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f(ctx)
        print(module)


def run_with_insertion_point_v2(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


def run_with_insertion_point_v3(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f(module)
        print(module)
    return f


def run_with_insertion_point_v4(f):
    print("\nTEST:", f.__name__)
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        module = Module.create()
        with InsertionPoint(module.body):
            f()
    return f


def run_apply_patterns(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with InsertionPoint(sequence.body):
                apply = transform.ApplyPatternsOp(sequence.bodyTarget)
                with InsertionPoint(apply.patterns):
                    f()
                transform.YieldOp()
        print("\nTEST:", f.__name__)
        print(module)
    return f


def run_transform_tensor_ext(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print(module)
    return f


def run_transform_structured_ext(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        module.operation.verify()
        print(module)
    return f


def run_construct_and_print_in_module(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            module = f(module)
        if module is not None:
            print(module)
    return f


TEST_MODULES = [
    ("execution_engine", run),
    ("pass_manager", run),
    ("dialects/affine", run_with_insertion_point_v2),
    ("dialects/func", run_with_insertion_point_v2),
    ("dialects/arith_dialect", run),
    ("dialects/arith_llvm", run),
    ("dialects/async_dialect", run),
    ("dialects/builtin", run),
    ("dialects/cf", run_with_insertion_point_v4),
    ("dialects/complex_dialect", run),
    ("dialects/func", run_with_insertion_point_v2),
    ("dialects/index_dialect", run_with_insertion_point),
    ("dialects/llvm", run_with_insertion_point_v2),
    ("dialects/math_dialect", run),
    ("dialects/memref", run),
    ("dialects/ml_program", run_with_insertion_point_v2),
    ("dialects/nvgpu", run_with_insertion_point_v2),
    ("dialects/nvvm", run_with_insertion_point_v2),
    ("dialects/ods_helpers", run),
    ("dialects/openmp_ops", run_with_insertion_point_v2),
    ("dialects/pdl_ops", run_with_insertion_point_v2),
    # ("dialects/python_test", run),  # TODO: Need to pass pybind11 or nanobind argv
    ("dialects/quant", run),
    ("dialects/rocdl", run_with_insertion_point_v2),
    ("dialects/scf", run_with_insertion_point_v2),
    ("dialects/shape", run),
    ("dialects/spirv_dialect", run),
    ("dialects/tensor", run),
    # ("dialects/tosa", ),  # Nothing to test
    ("dialects/transform_bufferization_ext", run_with_insertion_point_v2),
    # ("dialects/transform_extras", ),  # Needs a more complicated execution schema
    ("dialects/transform_gpu_ext", run_transform_tensor_ext),
    (
        "dialects/transform_interpreter",
        run_with_context_and_location,
        ["print_", "transform_options", "failed", "include"],
    ),
    (
        "dialects/transform_loop_ext",
        run_with_insertion_point_v2,
        ["loopOutline"],
    ),
    ("dialects/transform_memref_ext", run_with_insertion_point_v2),
    ("dialects/transform_nvgpu_ext", run_with_insertion_point_v2),
    ("dialects/transform_sparse_tensor_ext", run_transform_tensor_ext),
    ("dialects/transform_structured_ext", run_transform_structured_ext),
    ("dialects/transform_tensor_ext", run_transform_tensor_ext),
    (
        "dialects/transform_vector_ext",
        run_apply_patterns,
        ["configurable_patterns"],
    ),
    ("dialects/transform", run_with_insertion_point_v3),
    ("dialects/vector", run_with_context_and_location),
    ("dialects/gpu/dialect", run_with_context_and_location),
    ("dialects/gpu/module-to-binary-nvvm", run_with_context_and_location),
    ("dialects/gpu/module-to-binary-rocdl", run_with_context_and_location),
    ("dialects/linalg/ops", run),
    # TO ADD: No proper tests in this dialects/linalg/opsdsl/*
    # ("dialects/linalg/opsdsl/*", ...),
    ("dialects/sparse_tensor/dialect", run),
    ("dialects/sparse_tensor/passes", run),
    ("integration/dialects/pdl", run_construct_and_print_in_module),
    ("integration/dialects/transform", run_construct_and_print_in_module),
    ("integration/dialects/linalg/opsrun", run),
    ("ir/affine_expr", run),
    ("ir/affine_map", run),
    ("ir/array_attributes", run),
    ("ir/attributes", run),
    ("ir/blocks", run),
    ("ir/builtin_types", run),
    ("ir/context_managers", run),
    ("ir/debug", run),
    ("ir/diagnostic_handler", run),
    ("ir/dialects", run),
    ("ir/exception", run),
    ("ir/insertion_point", run),
    ("ir/integer_set", run),
    ("ir/location", run),
    ("ir/module", run),
    ("ir/operation", run),
    ("ir/symbol_table", run),
    ("ir/value", run),
]

TESTS_TO_SKIP = [
    "test_execution_engine__testNanoTime_multi_threaded",  # testNanoTime can't run in multiple threads, even with GIL
    "test_execution_engine__testSharedLibLoad_multi_threaded",  # testSharedLibLoad can't run in multiple threads, even with GIL
    "test_dialects_arith_dialect__testArithValue_multi_threaded",  # RuntimeError: Value caster is already registered: <class 'dialects/arith_dialect.testArithValue.<locals>.ArithValue'>, even with GIL
    "test_ir_dialects__testAppendPrefixSearchPath_multi_threaded",  # PyGlobals::setDialectSearchPrefixes is not thread-safe, even with GIL. Strange usage of static PyGlobals vs python exposed _cext.globals
    "test_ir_value__testValueCasters_multi_threaded",  # RuntimeError: Value caster is already registered: <function testValueCasters.<locals>.dont_cast_int, even with GIL
    # tests indirectly calling thread-unsafe llvm::raw_ostream
    "test_execution_engine__testInvalidModule_multi_threaded",  # mlirExecutionEngineCreate calls thread-unsafe llvm::raw_ostream
    "test_pass_manager__testPrintIrAfterAll_multi_threaded",  # IRPrinterInstrumentation::runAfterPass calls thread-unsafe llvm::raw_ostream
    "test_pass_manager__testPrintIrBeforeAndAfterAll_multi_threaded",  # IRPrinterInstrumentation::runBeforePass calls thread-unsafe llvm::raw_ostream
    "test_pass_manager__testPrintIrLargeLimitElements_multi_threaded",  # IRPrinterInstrumentation::runAfterPass calls thread-unsafe llvm::raw_ostream
    "test_pass_manager__testPrintIrTree_multi_threaded",  # IRPrinterInstrumentation::runAfterPass calls thread-unsafe llvm::raw_ostream
    "test_pass_manager__testRunPipeline_multi_threaded",  # PrintOpStatsPass::printSummary calls thread-unsafe llvm::raw_ostream
    "test_dialects_transform_interpreter__include_multi_threaded",  # mlir::transform::PrintOp::apply(mlir::transform::TransformRewriter...) calls thread-unsafe llvm::raw_ostream
    "test_dialects_transform_interpreter__transform_options_multi_threaded",  # mlir::transform::PrintOp::apply(mlir::transform::TransformRewriter...) calls thread-unsafe llvm::raw_ostream
    "test_dialects_transform_interpreter__print_self_multi_threaded",  # mlir::transform::PrintOp::apply(mlir::transform::TransformRewriter...) call thread-unsafe llvm::raw_ostream
    "test_ir_diagnostic_handler__testDiagnosticCallbackException_multi_threaded",  # mlirEmitError calls thread-unsafe llvm::raw_ostream
    "test_ir_module__testParseSuccess_multi_threaded",  # mlirOperationDump calls thread-unsafe llvm::raw_ostream
    # False-positive TSAN detected race in llvm::RuntimeDyldELF::registerEHFrames()
    # Details: https://github.com/llvm/llvm-project/pull/107103/files#r1905726947
    "test_execution_engine__testCapsule_multi_threaded",
    "test_execution_engine__testDumpToObjectFile_multi_threaded",
]

TESTS_TO_XFAIL = [
    # execution_engine tests:
    # - ctypes related data-races: https://github.com/python/cpython/issues/127945
    "test_execution_engine__testBF16Memref_multi_threaded",
    "test_execution_engine__testBasicCallback_multi_threaded",
    "test_execution_engine__testComplexMemrefAdd_multi_threaded",
    "test_execution_engine__testComplexUnrankedMemrefAdd_multi_threaded",
    "test_execution_engine__testDynamicMemrefAdd2D_multi_threaded",
    "test_execution_engine__testF16MemrefAdd_multi_threaded",
    "test_execution_engine__testF8E5M2Memref_multi_threaded",
    "test_execution_engine__testInvokeFloatAdd_multi_threaded",
    "test_execution_engine__testInvokeVoid_multi_threaded",  # a ctypes race
    "test_execution_engine__testMemrefAdd_multi_threaded",
    "test_execution_engine__testRankedMemRefCallback_multi_threaded",
    "test_execution_engine__testRankedMemRefWithOffsetCallback_multi_threaded",
    "test_execution_engine__testUnrankedMemRefCallback_multi_threaded",
    "test_execution_engine__testUnrankedMemRefWithOffsetCallback_multi_threaded",
    # dialects tests
    "test_dialects_memref__testSubViewOpInferReturnTypeExtensiveSlicing_multi_threaded",  # Related to ctypes data races
    "test_dialects_transform_interpreter__print_other_multi_threaded",  # Fatal Python error: Aborted or mlir::transform::PrintOp::apply(mlir::transform::TransformRewriter...) is not thread-safe
    "test_dialects_gpu_module-to-binary-rocdl__testGPUToASMBin_multi_threaded",  # Due to global llvm-project/llvm/lib/Target/AMDGPU/GCNSchedStrategy.cpp::GCNTrackers variable mutation
    "test_dialects_gpu_module-to-binary-nvvm__testGPUToASMBin_multi_threaded",
    "test_dialects_gpu_module-to-binary-nvvm__testGPUToLLVMBin_multi_threaded",
    "test_dialects_gpu_module-to-binary-rocdl__testGPUToLLVMBin_multi_threaded",
    # integration tests
    "test_integration_dialects_linalg_opsrun__test_elemwise_builtin_multi_threaded",  # Related to ctypes data races
    "test_integration_dialects_linalg_opsrun__test_elemwise_generic_multi_threaded",  # Related to ctypes data races
    "test_integration_dialects_linalg_opsrun__test_fill_builtin_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_fill_generic_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_fill_rng_builtin_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_fill_rng_generic_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_max_pooling_builtin_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_max_pooling_generic_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_min_pooling_builtin_multi_threaded",  # ctypes
    "test_integration_dialects_linalg_opsrun__test_min_pooling_generic_multi_threaded",  # ctypes
]


def add_existing_tests(test_modules, test_prefix: str = "_original_test"):
    def decorator(test_cls):
        this_folder = Path(__file__).parent.absolute()
        test_cls.output_folder = tempfile.TemporaryDirectory()
        output_folder = Path(test_cls.output_folder.name)

        for test_mod_info in test_modules:
            assert isinstance(test_mod_info, tuple) and len(test_mod_info) in (2, 3)
            if len(test_mod_info) == 2:
                test_module_name, exec_fn = test_mod_info
                test_pattern = None
            else:
                test_module_name, exec_fn, test_pattern = test_mod_info

            src_filepath = this_folder / f"{test_module_name}.py"
            dst_filepath = (output_folder / f"{test_module_name}.py").absolute()
            if not dst_filepath.parent.exists():
                dst_filepath.parent.mkdir(parents=True)
            copy_and_update(src_filepath, dst_filepath)
            test_mod = import_from_path(test_module_name, dst_filepath)
            for attr_name in dir(test_mod):
                is_test_fn = test_pattern is None and attr_name.startswith("test")
                is_test_fn |= test_pattern is not None and any(
                    [p in attr_name for p in test_pattern]
                )
                if is_test_fn:
                    obj = getattr(test_mod, attr_name)
                    if callable(obj):
                        test_name = f"{test_prefix}_{test_module_name.replace('/', '_')}__{attr_name}"

                        def wrapped_test_fn(
                            self, *args, __test_fn__=obj, __exec_fn__=exec_fn, **kwargs
                        ):
                            __exec_fn__(__test_fn__)

                        setattr(test_cls, test_name, wrapped_test_fn)
        return test_cls

    return decorator


@contextmanager
def _capture_output(fp):
    # Inspired from jax test_utils.py capture_stderr method
    # ``None`` means nothing has not been captured yet.
    captured = None

    def get_output() -> str:
        if captured is None:
            raise ValueError("get_output() called while the context is active.")
        return captured

    with tempfile.NamedTemporaryFile(mode="w+", encoding="utf-8") as f:
        original_fd = os.dup(fp.fileno())
        os.dup2(f.fileno(), fp.fileno())
        try:
            yield get_output
        finally:
            # Python also has its own buffers, make sure everything is flushed.
            fp.flush()
            os.fsync(fp.fileno())
            f.seek(0)
            captured = f.read()
            os.dup2(original_fd, fp.fileno())


capture_stdout = partial(_capture_output, sys.stdout)
capture_stderr = partial(_capture_output, sys.stderr)


def multi_threaded(
    num_workers: int,
    num_runs: int = 5,
    skip_tests: Optional[List[str]] = None,
    xfail_tests: Optional[List[str]] = None,
    test_prefix: str = "_original_test",
    multithreaded_test_postfix: str = "_multi_threaded",
):
    """Decorator that runs a test in a multi-threaded environment."""

    def decorator(test_cls):
        for name, test_fn in test_cls.__dict__.copy().items():
            if not (name.startswith(test_prefix) and callable(test_fn)):
                continue

            name = f"test{name[len(test_prefix):]}"
            if skip_tests is not None:
                if any(
                    test_name.replace(multithreaded_test_postfix, "") in name
                    for test_name in skip_tests
                ):
                    continue

            def multi_threaded_test_fn(self, *args, __test_fn__=test_fn, **kwargs):
                with capture_stdout(), capture_stderr() as get_output:
                    barrier = threading.Barrier(num_workers)

                    def closure():
                        barrier.wait()
                        for _ in range(num_runs):
                            __test_fn__(self, *args, **kwargs)

                    with concurrent.futures.ThreadPoolExecutor(
                        max_workers=num_workers
                    ) as executor:
                        futures = []
                        for _ in range(num_workers):
                            futures.append(executor.submit(closure))
                        # We should call future.result() to re-raise an exception if test has
                        # failed
                        assert len(list(f.result() for f in futures)) == num_workers

                    gc.collect()
                    assert Context._get_live_count() == 0

                captured = get_output()
                if len(captured) > 0 and "ThreadSanitizer" in captured:
                    raise RuntimeError(
                        f"ThreadSanitizer reported warnings:\n{captured}"
                    )

            test_new_name = f"{name}{multithreaded_test_postfix}"
            if xfail_tests is not None and test_new_name in xfail_tests:
                multi_threaded_test_fn = unittest.expectedFailure(
                    multi_threaded_test_fn
                )

            setattr(test_cls, test_new_name, multi_threaded_test_fn)

        return test_cls

    return decorator


@multi_threaded(
    num_workers=10,
    num_runs=20,
    skip_tests=TESTS_TO_SKIP,
    xfail_tests=TESTS_TO_XFAIL,
)
@add_existing_tests(test_modules=TEST_MODULES, test_prefix="_original_test")
class TestAllMultiThreaded(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        if hasattr(cls, "output_folder"):
            cls.output_folder.cleanup()

    def _original_test_create_context(self):
        with Context() as ctx:
            print(ctx._get_live_count())
            print(ctx._get_live_module_count())
            print(ctx._get_live_operation_count())
            print(ctx._get_live_operation_objects())
            print(ctx._get_context_again() is ctx)
            print(ctx._clear_live_operations())

    def _original_test_create_module_with_consts(self):
        py_values = [123, 234, 345]
        with Context() as ctx:
            module = Module.create(loc=Location.file("foo.txt", 0, 0))

            dtype = IntegerType.get_signless(64)
            with InsertionPoint(module.body), Location.name("a"):
                arith.constant(dtype, py_values[0])

            with InsertionPoint(module.body), Location.name("b"):
                arith.constant(dtype, py_values[1])

            with InsertionPoint(module.body), Location.name("c"):
                arith.constant(dtype, py_values[2])


if __name__ == "__main__":
    # Do not run the tests on CPython with GIL
    if hasattr(sys, "_is_gil_enabled") and not sys._is_gil_enabled():
        unittest.main()



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class InvalidArgsCommandTestCase(TestBase):
    @no_debug_info_test
    def test_script_add(self):
        self.expect(
            "command script add 1 2",
            error=True,
            substrs=["Path component: '1' not found"],
        )

        self.expect(
            "command script add",
            error=True,
            substrs=["'command script add' requires at least one argument"],
        )

    @no_debug_info_test
    def test_script_clear(self):
        self.expect(
            "command script clear f",
            error=True,
            substrs=["'command script clear' doesn't take any arguments"],
        )

    @no_debug_info_test
    def test_script_list(self):
        self.expect(
            "command script list f",
            error=True,
            substrs=["'command script list' doesn't take any arguments"],
        )

    @no_debug_info_test
    def test_script_import(self):
        self.expect(
            "command script import",
            error=True,
            substrs=["command script import needs one or more arguments"],
        )

    @no_debug_info_test
    def test_alias(self):
        self.expect(
            "command alias",
            error=True,
            substrs=["'command alias' requires at least two arguments"],
        )

        self.expect(
            "command alias blub foo",
            error=True,
            substrs=[
                "error: invalid command given to 'command alias'. 'foo' does not begin with a valid command.  No alias created."
            ],
        )

    @no_debug_info_test
    def test_unalias(self):
        self.expect(
            "command unalias",
            error=True,
            substrs=["must call 'unalias' with a valid alias"],
        )

    @no_debug_info_test
    def test_delete(self):
        self.expect(
            "command delete",
            error=True,
            substrs=["must call 'command delete' with one or more valid user"],
        )

    @no_debug_info_test
    def test_regex(self):
        self.expect(
            "command regex",
            error=True,
            substrs=["usage: 'command regex <command-name> "],
        )

    @no_debug_info_test
    def test_source(self):
        self.expect(
            "command source",
            error=True,
            substrs=[
                "'command source' takes exactly one executable filename argument."
            ],
        )



"""
Tests C99's flexible array members.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        self.expect_var_path("c->flexible", type="char[]", summary='"contents"')
        # self.expect_var_path("sc->flexible", type="signed char[]", summary='"contents"')
        self.expect_var_path(
            "uc->flexible", type="unsigned char[]", summary='"contents"'
        )
        # TODO: Make this work
        self.expect("expr c->flexible", error=True, substrs=["incomplete", "char[]"])
        self.expect(
            "expr sc->flexible", error=True, substrs=["incomplete", "signed char[]"]
        )
        self.expect(
            "expr uc->flexible", error=True, substrs=["incomplete", "unsigned char[]"]
        )



"""
Test the use of setjmp/longjmp for non-local goto operations in a single-threaded inferior.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LongjmpTestCase(TestBase):
    @skipIfDarwin  # llvm.org/pr16769: LLDB on Mac OS X dies in function ReadRegisterBytes in GDBRemoteRegisterContext.cpp
    @skipIfWindows # Test is flaky on Windows
    @expectedFlakeyNetBSD
    def test_step_out(self):
        """Test stepping when the inferior calls setjmp/longjmp, in particular, thread step-out."""
        self.build()
        self.step_out()

    @skipIfDarwin  # llvm.org/pr16769: LLDB on Mac OS X dies in function ReadRegisterBytes in GDBRemoteRegisterContext.cpp
    @skipIfWindows # Test is flaky on Windows
    @skipIfNetBSD
    def test_step_over(self):
        """Test stepping when the inferior calls setjmp/longjmp, in particular, thread step-over a longjmp."""
        self.build()
        self.step_over()

    @skipIfDarwin  # llvm.org/pr16769: LLDB on Mac OS X dies in function ReadRegisterBytes in GDBRemoteRegisterContext.cpp
    @skipIfWindows # Test is flaky on Windows
    @expectedFlakeyNetBSD
    def test_step_back_out(self):
        """Test stepping when the inferior calls setjmp/longjmp, in particular, thread step-out after thread step-in."""
        self.build()
        self.step_back_out()

    def start_test(self, symbol):
        exe = self.getBuildArtifact("a.out")

        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main().
        lldbutil.run_break_set_by_symbol(self, symbol, num_expected_locations=-1)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    def check_status(self):
        # Note: Depending on the generated mapping of DWARF to assembly,
        # the process may have stopped or exited.
        self.expect(
            "process status",
            PROCESS_STOPPED,
            patterns=["Process .* exited with status = 0"],
        )

    def step_out(self):
        self.start_test("do_jump")
        self.runCmd("thread step-out", RUN_SUCCEEDED)
        self.check_status()

    def step_over(self):
        self.start_test("do_jump")
        self.runCmd("thread step-over", RUN_SUCCEEDED)
        self.runCmd("thread step-over", RUN_SUCCEEDED)
        self.check_status()

    def step_back_out(self):
        self.start_test("main")

        self.runCmd("thread step-over", RUN_SUCCEEDED)
        self.runCmd("thread step-in", RUN_SUCCEEDED)
        self.runCmd("thread step-out", RUN_SUCCEEDED)
        self.check_status()



import os
import pathlib
import platform
import subprocess
import sys
import itertools

import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbplatformutil as lldbplatformutil
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test import configuration
from lldbsuite.test_event import build_exception
from lldbsuite.support import seven


class Builder:
    def getArchitecture(self):
        """Returns the architecture in effect the test suite is running with."""
        return configuration.arch if configuration.arch else ""

    def getCompiler(self):
        """Returns the compiler in effect the test suite is running with."""
        compiler = configuration.compiler if configuration.compiler else "clang"
        compiler = lldbutil.which(compiler)
        return os.path.abspath(compiler)

    def getTriple(self, arch):
        """Returns the triple for the given architecture or None."""
        return configuration.triple

    def getExtraMakeArgs(self):
        """
        Helper function to return extra argumentsfor the make system. This
        method is meant to be overridden by platform specific builders.
        """
        return []

    def getArchCFlags(self, architecture):
        """Returns the ARCH_CFLAGS for the make system."""
        triple = self.getTriple(architecture)
        if triple:
            return ["ARCH_CFLAGS=-target {}".format(triple)]
        return []

    def getMake(self, test_subdir, test_name):
        """Returns the invocation for GNU make.
        The first argument is a tuple of the relative path to the testcase
        and its filename stem."""
        # Construct the base make invocation.
        lldb_test = os.environ["LLDB_TEST"]
        if not (
            lldb_test
            and configuration.test_build_dir
            and test_subdir
            and test_name
            and (not os.path.isabs(test_subdir))
        ):
            raise Exception("Could not derive test directories")
        build_dir = os.path.join(configuration.test_build_dir, test_subdir, test_name)
        src_dir = os.path.join(configuration.test_src_root, test_subdir)
        # This is a bit of a hack to make inline testcases work.
        makefile = os.path.join(src_dir, "Makefile")
        if not os.path.isfile(makefile):
            makefile = os.path.join(build_dir, "Makefile")
        return [
            configuration.make_path,
            "VPATH=" + src_dir,
            "-C",
            build_dir,
            "-I",
            src_dir,
            "-I",
            os.path.join(lldb_test, "make"),
            "-f",
            makefile,
        ]

    def getCmdLine(self, d):
        """
        Helper function to return a command line argument string used for the
        make system.
        """

        # If d is None or an empty mapping, just return an empty list.
        if not d:
            return []

        def setOrAppendVariable(k, v):
            append_vars = ["CFLAGS", "CFLAGS_EXTRAS", "LD_EXTRAS"]
            if k in append_vars and k in os.environ:
                v = os.environ[k] + " " + v
            return "%s=%s" % (k, v)

        cmdline = [setOrAppendVariable(k, v) for k, v in list(d.items())]

        return cmdline

    def getArchSpec(self, architecture):
        """
        Helper function to return the key-value string to specify the architecture
        used for the make system.
        """
        return ["ARCH=" + architecture] if architecture else []

    def getToolchainSpec(self, compiler):
        """
        Helper function to return the key-value strings to specify the toolchain
        used for the make system.
        """
        cc = compiler if compiler else None
        if not cc and configuration.compiler:
            cc = configuration.compiler

        if not cc:
            return []

        exe_ext = ""
        if lldbplatformutil.getHostPlatform() == "windows":
            exe_ext = ".exe"

        cc = cc.strip()
        cc_path = pathlib.Path(cc)

        # We can get CC compiler string in the following formats:
        #  [<tool>] <compiler>    - such as 'xrun clang', 'xrun /usr/bin/clang' & etc
        #
        # Where <compiler> could contain the following parts:
        #   <simple-name>[.<exe-ext>]                           - sucn as 'clang', 'clang.exe' ('clang-cl.exe'?)
        #   <target-triple>-<simple-name>[.<exe-ext>]           - such as 'armv7-linux-gnueabi-gcc'
        #   <path>/<simple-name>[.<exe-ext>]                    - such as '/usr/bin/clang', 'c:\path\to\compiler\clang,exe'
        #   <path>/<target-triple>-<simple-name>[.<exe-ext>]    - such as '/usr/bin/clang', 'c:\path\to\compiler\clang,exe'

        cc_ext = cc_path.suffix
        # Compiler name without extension
        cc_name = cc_path.stem.split(" ")[-1]

        # A kind of compiler (canonical name): clang, gcc, cc & etc.
        cc_type = cc_name
        # A triple prefix of compiler name: <armv7-none-linux-gnu->gcc
        cc_prefix = ""
        if "clang-cl" not in cc_name:
            cc_name_parts = cc_name.split("-")
            cc_type = cc_name_parts[-1]
            if len(cc_name_parts) > 1:
                cc_prefix = "-".join(cc_name_parts[:-1]) + "-"

        # A kind of C++ compiler.
        cxx_types = {
            "icc": "icpc",
            "gcc": "g++",
            "cc": "c++",
            "clang": "clang++",
            "xcrun clang": "xcrun clang++",
        }
        # Determine the C++ compiler based on the given compiler path/command.
        cxx_type = cxx_types.get(compiler)
        if cxx_type is None:
            # If that did not work, then use the inferred cc_type.
            cxx_type = cxx_types.get(cc_type, cxx_type)

        cc_dir = cc_path.parent

        def getToolchainUtil(util_name):
            return os.path.join(configuration.llvm_tools_dir, util_name + exe_ext)

        cxx = cc_dir / (cc_prefix + cxx_type + cc_ext)

        util_names = {
            "OBJCOPY": "objcopy",
            "STRIP": "strip",
            "ARCHIVER": "ar",
            "DWP": "dwp",
        }
        utils = []

        # Required by API TestBSDArchives.py tests.
        if not os.getenv("LLVM_AR"):
            utils.extend(["LLVM_AR=%s" % getToolchainUtil("llvm-ar")])

        if cc_type in ["clang", "cc", "gcc"]:
            util_paths = {}
            # Assembly a toolchain side tool cmd based on passed CC.
            for var, name in util_names.items():
                # Do not override explicity specified tool from the cmd line.
                if not os.getenv(var):
                    util_paths[var] = getToolchainUtil("llvm-" + name)
                else:
                    util_paths[var] = os.getenv(var)
            utils.extend(["AR=%s" % util_paths["ARCHIVER"]])

            # Look for llvm-dwp or gnu dwp
            if not lldbutil.which(util_paths["DWP"]):
                util_paths["DWP"] = getToolchainUtil("llvm-dwp")
            if not lldbutil.which(util_paths["DWP"]):
                util_paths["DWP"] = lldbutil.which("llvm-dwp")
            if not util_paths["DWP"]:
                util_paths["DWP"] = lldbutil.which("dwp")
                if not util_paths["DWP"]:
                    del util_paths["DWP"]

            if lldbplatformutil.platformIsDarwin():
                util_paths["STRIP"] = seven.get_command_output("xcrun -f strip")

            for var, path in util_paths.items():
                utils.append("%s=%s" % (var, path))

        if lldbplatformutil.platformIsDarwin():
            utils.extend(["AR=%slibtool" % os.getenv("CROSS_COMPILE", "")])

        return [
            "CC=%s" % cc,
            "CC_TYPE=%s" % cc_type,
            "CXX=%s" % cxx,
        ] + utils

    def getSDKRootSpec(self):
        """
        Helper function to return the key-value string to specify the SDK root
        used for the make system.
        """
        if configuration.sdkroot:
            return ["SDKROOT={}".format(configuration.sdkroot)]
        return []

    def getModuleCacheSpec(self):
        """
        Helper function to return the key-value string to specify the clang
        module cache used for the make system.
        """
        if configuration.clang_module_cache_dir:
            return [
                "CLANG_MODULE_CACHE_DIR={}".format(configuration.clang_module_cache_dir)
            ]
        return []

    def getLibCxxArgs(self):
        if configuration.libcxx_include_dir and configuration.libcxx_library_dir:
            libcpp_args = [
                "LIBCPP_INCLUDE_DIR={}".format(configuration.libcxx_include_dir),
                "LIBCPP_LIBRARY_DIR={}".format(configuration.libcxx_library_dir),
            ]
            if configuration.libcxx_include_target_dir:
                libcpp_args.append(
                    "LIBCPP_INCLUDE_TARGET_DIR={}".format(
                        configuration.libcxx_include_target_dir
                    )
                )
            return libcpp_args
        return []

    def getLLDBObjRoot(self):
        return ["LLDB_OBJ_ROOT={}".format(configuration.lldb_obj_root)]

    def _getDebugInfoArgs(self, debug_info):
        if debug_info is None:
            return []

        debug_options = debug_info if isinstance(debug_info, list) else [debug_info]
        option_flags = {
            "dwarf": {"MAKE_DSYM": "NO"},
            "dwo": {"MAKE_DSYM": "NO", "MAKE_DWO": "YES"},
            "gmodules": {"MAKE_DSYM": "NO", "MAKE_GMODULES": "YES"},
            "debug_names": {"MAKE_DEBUG_NAMES": "YES"},
            "dwp": {"MAKE_DSYM": "NO", "MAKE_DWP": "YES"},
            "pdb": {"MAKE_PDB": "YES"},
        }

        # Collect all flags, with later options overriding earlier ones
        flags = {}

        for option in debug_options:
            if not option or option not in option_flags:
                return None  # Invalid options
            flags.update(option_flags[option])

        return [f"{key}={value}" for key, value in flags.items()]

    def getBuildCommand(
        self,
        debug_info,
        architecture=None,
        compiler=None,
        dictionary=None,
        testdir=None,
        testname=None,
        make_targets=None,
    ):
        debug_info_args = self._getDebugInfoArgs(debug_info)
        if debug_info_args is None:
            return None
        if make_targets is None:
            make_targets = ["all"]
        command_parts = [
            self.getMake(testdir, testname),
            debug_info_args,
            make_targets,
            self.getArchCFlags(architecture),
            self.getArchSpec(architecture),
            self.getToolchainSpec(compiler),
            self.getExtraMakeArgs(),
            self.getSDKRootSpec(),
            self.getModuleCacheSpec(),
            self.getLibCxxArgs(),
            self.getLLDBObjRoot(),
            self.getCmdLine(dictionary),
        ]
        command = list(itertools.chain(*command_parts))

        return command

    def cleanup(self, dictionary=None):
        """Perform a platform-specific cleanup after the test."""
        return True



"""
Use lldb Python API to test dynamic values in C++
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DynamicValueTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.

        self.do_something_line = line_number(
            "pass-to-base.cpp", "// Break here in doSomething."
        )
        self.main_first_call_line = line_number(
            "pass-to-base.cpp",
            "// Break here and get real addresses of myB and otherB.",
        )
        self.main_second_call_line = line_number(
            "pass-to-base.cpp", "// Break here and get real address of reallyA."
        )

    @add_test_categories(["pyapi"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24663")
    def test_get_dynamic_vals(self):
        """Test fetching C++ dynamic values from pointers & references."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoints:

        do_something_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.do_something_line
        )
        self.assertTrue(do_something_bpt, VALID_BREAKPOINT)

        first_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_first_call_line
        )
        self.assertTrue(first_call_bpt, VALID_BREAKPOINT)

        second_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_second_call_line
        )
        self.assertTrue(second_call_bpt, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, first_call_bpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)

        # Now find the dynamic addresses of myB and otherB so we can compare them
        # with the dynamic values we get in doSomething:

        use_dynamic = lldb.eDynamicCanRunTarget
        no_dynamic = lldb.eNoDynamicValues

        myB = frame.FindVariable("myB", no_dynamic)
        self.assertTrue(myB)
        myB_loc = int(myB.GetLocation(), 16)

        otherB = frame.FindVariable("otherB", no_dynamic)
        self.assertTrue(otherB)
        otherB_loc = int(otherB.GetLocation(), 16)

        # Okay now run to doSomething:

        threads = lldbutil.continue_to_breakpoint(process, do_something_bpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)

        # Get "this" using FindVariable:

        this_static = frame.FindVariable("this", no_dynamic)
        this_dynamic = frame.FindVariable("this", use_dynamic)
        self.examine_value_object_of_this_ptr(this_static, this_dynamic, myB_loc)

        # Now make sure that the "GetDynamicValue" works:
        # This doesn't work currently because we can't get dynamic values from
        # ConstResult objects.
        fetched_dynamic_value = this_static.GetDynamicValue(use_dynamic)
        self.examine_value_object_of_this_ptr(
            this_static, fetched_dynamic_value, myB_loc
        )

        # And conversely that the GetDynamicValue() interface also works:
        fetched_static_value = this_dynamic.GetStaticValue()
        self.examine_value_object_of_this_ptr(
            fetched_static_value, this_dynamic, myB_loc
        )

        # Get "this" using FindValue, make sure that works too:
        this_static = frame.FindValue(
            "this", lldb.eValueTypeVariableArgument, no_dynamic
        )
        this_dynamic = frame.FindValue(
            "this", lldb.eValueTypeVariableArgument, use_dynamic
        )
        self.examine_value_object_of_this_ptr(this_static, this_dynamic, myB_loc)

        # Get "this" using the EvaluateExpression:
        this_static = frame.EvaluateExpression("this", False)
        this_dynamic = frame.EvaluateExpression("this", True)
        self.examine_value_object_of_this_ptr(this_static, this_dynamic, myB_loc)

        # The "frame var" code uses another path to get into children, so let's
        # make sure that works as well:

        self.expect(
            "frame var -d run-target --ptr-depth=2 --show-types anotherA.m_client_A",
            "frame var finds its way into a child member",
            patterns=[r"\(B \*\)"],
        )

        # Now make sure we also get it right for a reference as well:

        anotherA_static = frame.FindVariable("anotherA", False)
        self.assertTrue(anotherA_static)
        anotherA_static_addr = int(anotherA_static.GetValue(), 16)

        anotherA_dynamic = frame.FindVariable("anotherA", True)
        self.assertTrue(anotherA_dynamic)
        anotherA_dynamic_addr = int(anotherA_dynamic.GetValue(), 16)
        anotherA_dynamic_typename = anotherA_dynamic.GetTypeName()
        self.assertNotEqual(anotherA_dynamic_typename.find("B"), -1)

        self.assertLess(anotherA_dynamic_addr, anotherA_static_addr)

        anotherA_m_b_value_dynamic = anotherA_dynamic.GetChildMemberWithName(
            "m_b_value", True
        )
        self.assertTrue(anotherA_m_b_value_dynamic)
        anotherA_m_b_val = int(anotherA_m_b_value_dynamic.GetValue(), 10)
        self.assertEqual(anotherA_m_b_val, 300)

        anotherA_m_b_value_static = anotherA_static.GetChildMemberWithName(
            "m_b_value", True
        )
        self.assertFalse(anotherA_m_b_value_static)

        # Okay, now continue again, and when we hit the second breakpoint in
        # main

        threads = lldbutil.continue_to_breakpoint(process, second_call_bpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)
        reallyA_value = frame.FindVariable("reallyA", False)
        self.assertTrue(reallyA_value)
        reallyA_loc = int(reallyA_value.GetLocation(), 16)

        # Continue to doSomething again, and make sure we get the right value for anotherA,
        # which this time around is just an "A".

        threads = lldbutil.continue_to_breakpoint(process, do_something_bpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)
        anotherA_value = frame.FindVariable("anotherA", True)
        self.assertTrue(anotherA_value)
        anotherA_loc = int(anotherA_value.GetValue(), 16)
        self.assertEqual(anotherA_loc, reallyA_loc)
        self.assertEqual(anotherA_value.GetTypeName().find("B"), -1)

        # Finally do the same with a B in an anonymous namespace.
        threads = lldbutil.continue_to_breakpoint(process, do_something_bpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)
        anotherA_value = frame.FindVariable("anotherA", use_dynamic)
        self.assertTrue(anotherA_value)
        self.assertIn("B", anotherA_value.GetTypeName())
        anon_b_value = anotherA_value.GetChildMemberWithName("m_anon_b_value")
        self.assertTrue(anon_b_value)
        self.assertEqual(anon_b_value.GetValueAsSigned(), 47)

    def examine_value_object_of_this_ptr(
        self, this_static, this_dynamic, dynamic_location
    ):
        # Get "this" as its static value
        self.assertTrue(this_static)
        this_static_loc = int(this_static.GetValue(), 16)

        # Get "this" as its dynamic value

        self.assertTrue(this_dynamic)
        this_dynamic_typename = this_dynamic.GetTypeName()
        self.assertNotEqual(this_dynamic_typename.find("B"), -1)
        this_dynamic_loc = int(this_dynamic.GetValue(), 16)

        # Make sure we got the right address for "this"

        self.assertEqual(this_dynamic_loc, dynamic_location)

        # And that the static address is greater than the dynamic one

        self.assertGreater(this_static_loc, this_dynamic_loc)

        # Now read m_b_value which is only in the dynamic value:

        use_dynamic = lldb.eDynamicCanRunTarget
        no_dynamic = lldb.eNoDynamicValues

        this_dynamic_m_b_value = this_dynamic.GetChildMemberWithName(
            "m_b_value", use_dynamic
        )
        self.assertTrue(this_dynamic_m_b_value)

        m_b_value = int(this_dynamic_m_b_value.GetValue(), 0)
        self.assertEqual(m_b_value, 10)

        # Make sure it is not in the static version

        this_static_m_b_value = this_static.GetChildMemberWithName(
            "m_b_value", no_dynamic
        )
        self.assertFalse(this_static_m_b_value)

        # Okay, now let's make sure that we can get the dynamic type of a child
        # element:

        contained_auto_ptr = this_dynamic.GetChildMemberWithName(
            "m_client_A", use_dynamic
        )
        self.assertTrue(contained_auto_ptr)
        contained_b = contained_auto_ptr.GetChildMemberWithName("_M_ptr", use_dynamic)
        if not contained_b:
            contained_b = contained_auto_ptr.GetChildMemberWithName(
                "__ptr_", use_dynamic
            )
        self.assertTrue(contained_b)

        contained_b_static = contained_auto_ptr.GetChildMemberWithName(
            "_M_ptr", no_dynamic
        )
        if not contained_b_static:
            contained_b_static = contained_auto_ptr.GetChildMemberWithName(
                "__ptr_", no_dynamic
            )
        self.assertTrue(contained_b_static)

        contained_b_addr = int(contained_b.GetValue(), 16)
        contained_b_static_addr = int(contained_b_static.GetValue(), 16)

        self.assertLess(contained_b_addr, contained_b_static_addr)

    @no_debug_info_test
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24663")
    def test_from_forward_decl(self):
        """Test fetching C++ dynamic values forward-declared types. It's
        imperative that this is a separate test so that we don't end up parsing
        a definition of A from somewhere else."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "take_A")
        self.expect(
            "frame var -d run-target --ptr-depth=1 --show-types a",
            substrs=["(B *) a", "m_b_value = 10"],
        )

    @no_debug_info_test
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24663")
    @expectedFailureAll(archs=["arm$"]) # Minidump saving not implemented
    def test_from_core_file(self):
        """Test fetching C++ dynamic values from core files. Specifically, test
        that we can determine the dynamic type of the value if the core file
        does not contain the type vtable."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "take_A")

        # Get the address of our object and its vtable
        a = self.frame().FindVariable("a")
        self.assertSuccess(a.GetError())
        vtable = a.GetVTable()
        self.assertSuccess(vtable.GetError())
        a = a.GetValueAsAddress()
        vtable = vtable.GetValueAsAddress()

        # Create a core file which will only contain the memory region
        # containing `a`. The object is on the stack, so this will automatically
        # include the stack of the main thread.
        core = self.getBuildArtifact("a.dmp")
        options = lldb.SBSaveCoreOptions()
        options.SetPluginName("minidump")
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        options.SetOutputFile(lldb.SBFileSpec(core))
        region = lldb.SBMemoryRegionInfo()
        self.assertSuccess(self.process().GetMemoryRegionInfo(a, region))
        self.assertSuccess(options.AddMemoryRegionToSave(region))

        # Save the core file and load it.
        self.assertSuccess(self.process().SaveCore(options))
        self.process().Kill()
        error = lldb.SBError()
        self.target().LoadCore(core, error)
        self.assertSuccess(error)

        # Sanity check -- the process should be able to read the object but not
        # its vtable..
        self.process().ReadPointerFromMemory(a, error)
        self.assertSuccess(error)
        self.process().ReadPointerFromMemory(vtable, error)
        self.assertTrue(error.Fail())

        # .. but we should still be able to see the dynamic type by reading the
        # vtable from the executable file.
        self.expect(
            "frame var -d run-target --ptr-depth=1 --show-types a",
            substrs=["(B *) a", "m_b_value = 10"],
        )



from textwrap import dedent
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class MyResponder(MockGDBServerResponder):
    def qXferRead(self, obj, annex, offset, length):
        if annex == "target.xml":
            return (
                dedent(
                    """\
                <?xml version="1.0"?>
                  <target version="1.0">
                    <architecture>aarch64</architecture>
                    <feature name="org.gnu.gdb.aarch64.core">
                      <reg name="cpsr" regnum="33" bitsize="32"/>
                      <reg name="x0" regnum="0" bitsize="64"/>
                      <reg name="x1" bitsize="64"/>
                      <reg name="x2" bitsize="64"/>
                      <reg name="x3" bitsize="64"/>
                      <reg name="x4" bitsize="64"/>
                      <reg name="x5" bitsize="64"/>
                      <reg name="x6" bitsize="64"/>
                      <reg name="x7" bitsize="64"/>
                      <reg name="x8" bitsize="64"/>
                      <reg name="x9" bitsize="64"/>
                      <reg name="x10" bitsize="64"/>
                      <reg name="x11" bitsize="64"/>
                      <reg name="x12" bitsize="64"/>
                      <reg name="x13" bitsize="64"/>
                      <reg name="x14" bitsize="64"/>
                      <reg name="x15" bitsize="64"/>
                      <reg name="x16" bitsize="64"/>
                      <reg name="x17" bitsize="64"/>
                      <reg name="x18" bitsize="64"/>
                      <reg name="x19" bitsize="64"/>
                      <reg name="x20" bitsize="64"/>
                      <reg name="x21" bitsize="64"/>
                      <reg name="x22" bitsize="64"/>
                      <reg name="x23" bitsize="64"/>
                      <reg name="x24" bitsize="64"/>
                      <reg name="x25" bitsize="64"/>
                      <reg name="x26" bitsize="64"/>
                      <reg name="x27" bitsize="64"/>
                      <reg name="x28" bitsize="64"/>
                      <reg name="x29" bitsize="64"/>
                      <reg name="x30" bitsize="64"/>
                      <reg name="sp" bitsize="64"/>
                      <reg name="pc" bitsize="64"/>
                      <reg name="w0" bitsize="32" value_regnums="0" invalidate_regnums="0" regnum="34"/>
                      <reg name="w1" bitsize="32" value_regnums="1" invalidate_regnums="1"/>
                      <reg name="w2" bitsize="32" value_regnums="2" invalidate_regnums="2"/>
                      <reg name="w3" bitsize="32" value_regnums="3" invalidate_regnums="3"/>
                      <reg name="w4" bitsize="32" value_regnums="4" invalidate_regnums="4"/>
                      <reg name="w5" bitsize="32" value_regnums="5" invalidate_regnums="5"/>
                      <reg name="w6" bitsize="32" value_regnums="6" invalidate_regnums="6"/>
                      <reg name="w7" bitsize="32" value_regnums="7" invalidate_regnums="7"/>
                      <reg name="w8" bitsize="32" value_regnums="8" invalidate_regnums="8"/>
                      <reg name="w9" bitsize="32" value_regnums="9" invalidate_regnums="9"/>
                      <reg name="w10" bitsize="32" value_regnums="10" invalidate_regnums="10"/>
                      <reg name="w11" bitsize="32" value_regnums="11" invalidate_regnums="11"/>
                      <reg name="w12" bitsize="32" value_regnums="12" invalidate_regnums="12"/>
                      <reg name="w13" bitsize="32" value_regnums="13" invalidate_regnums="13"/>
                      <reg name="w14" bitsize="32" value_regnums="14" invalidate_regnums="14"/>
                      <reg name="w15" bitsize="32" value_regnums="15" invalidate_regnums="15"/>
                      <reg name="w16" bitsize="32" value_regnums="16" invalidate_regnums="16"/>
                      <reg name="w17" bitsize="32" value_regnums="17" invalidate_regnums="17"/>
                      <reg name="w18" bitsize="32" value_regnums="18" invalidate_regnums="18"/>
                      <reg name="w19" bitsize="32" value_regnums="19" invalidate_regnums="19"/>
                      <reg name="w20" bitsize="32" value_regnums="20" invalidate_regnums="20"/>
                      <reg name="w21" bitsize="32" value_regnums="21" invalidate_regnums="21"/>
                      <reg name="w22" bitsize="32" value_regnums="22" invalidate_regnums="22"/>
                      <reg name="w23" bitsize="32" value_regnums="23" invalidate_regnums="23"/>
                      <reg name="w24" bitsize="32" value_regnums="24" invalidate_regnums="24"/>
                      <reg name="w25" bitsize="32" value_regnums="25" invalidate_regnums="25"/>
                      <reg name="w26" bitsize="32" value_regnums="26" invalidate_regnums="26"/>
                      <reg name="w27" bitsize="32" value_regnums="27" invalidate_regnums="27"/>
                      <reg name="w28" bitsize="32" value_regnums="28" invalidate_regnums="28"/>
                    </feature>
                  </target>
                """
                ),
                False,
            )
        else:
            return (None,)

    def readRegister(self, regnum):
        return "E01"

    def readRegisters(self):
        return "20000000000000002000000000000000f0c154bfffff00005daa985a8fea0b48f0b954bfffff0000ad13cce570150b48380000000000000070456abfffff0000a700000000000000000000000000000001010101010101010000000000000000f0c154bfffff00000f2700000000000008e355bfffff0000080e55bfffff0000281041000000000010de61bfffff00005c05000000000000f0c154bfffff000090fcffffffff00008efcffffffff00008ffcffffffff00000000000000000000001000000000000090fcffffffff000000d06cbfffff0000f0c154bfffff00000100000000000000d0b954bfffff0000e407400000000000d0b954bfffff0000e40740000000000000100000"


class TestAArch64XMLRegOffsets(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("AArch64")
    def test_register_gpacket_offsets(self):
        """
        Test that we correctly associate the register info with the eh_frame
        register numbers.
        """

        target = self.createTarget("basic_eh_frame-aarch64.yaml")
        self.server.responder = MyResponder()

        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        registerSet = (
            process.GetThreadAtIndex(0)
            .GetFrameAtIndex(0)
            .GetRegisters()
            .GetValueAtIndex(0)
        )

        reg_val_dict = {
            "x0": 0x0000000000000020,
            "x1": 0x0000000000000020,
            "x2": 0x0000FFFFBF54C1F0,
            "x3": 0x480BEA8F5A98AA5D,
            "x4": 0x0000FFFFBF54B9F0,
            "x5": 0x480B1570E5CC13AD,
            "x6": 0x0000000000000038,
            "x7": 0x0000FFFFBF6A4570,
            "x8": 0x00000000000000A7,
            "x9": 0x0000000000000000,
            "x10": 0x0101010101010101,
            "x11": 0x0000000000000000,
            "x12": 0x0000FFFFBF54C1F0,
            "x13": 0x000000000000270F,
            "x14": 0x0000FFFFBF55E308,
            "x15": 0x0000FFFFBF550E08,
            "x16": 0x0000000000411028,
            "x17": 0x0000FFFFBF61DE10,
            "x18": 0x000000000000055C,
            "x19": 0x0000FFFFBF54C1F0,
            "x20": 0x0000FFFFFFFFFC90,
            "x21": 0x0000FFFFFFFFFC8E,
            "x22": 0x0000FFFFFFFFFC8F,
            "x23": 0x0000000000000000,
            "x24": 0x0000000000001000,
            "x25": 0x0000FFFFFFFFFC90,
            "x26": 0x0000FFFFBF6CD000,
            "x27": 0x0000FFFFBF54C1F0,
            "x28": 0x0000000000000001,
            "x29": 0x0000FFFFBF54B9D0,
            "x30": 0x00000000004007E4,
            "sp": 0x0000FFFFBF54B9D0,
            "pc": 0x00000000004007E4,
            "cpsr": 0x00001000,
            "w0": 0x00000020,
            "w1": 0x00000020,
            "w2": 0xBF54C1F0,
            "w3": 0x5A98AA5D,
            "w4": 0xBF54B9F0,
            "w5": 0xE5CC13AD,
            "w6": 0x00000038,
            "w7": 0xBF6A4570,
            "w8": 0x000000A7,
            "w9": 0x00000000,
            "w10": 0x01010101,
            "w11": 0x00000000,
            "w12": 0xBF54C1F0,
            "w13": 0x0000270F,
            "w14": 0xBF55E308,
            "w15": 0xBF550E08,
            "w16": 0x00411028,
            "w17": 0xBF61DE10,
            "w18": 0x0000055C,
            "w19": 0xBF54C1F0,
            "w20": 0xFFFFFC90,
            "w21": 0xFFFFFC8E,
            "w22": 0xFFFFFC8F,
            "w23": 0x00000000,
            "w24": 0x00001000,
            "w25": 0xFFFFFC90,
            "w26": 0xBF6CD000,
            "w27": 0xBF54C1F0,
            "w28": 0x00000001,
        }

        for reg in registerSet:
            self.assertEqual(reg.GetValueAsUnsigned(), reg_val_dict[reg.GetName()])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_file_close_invalid_arg(self):
        self.expect(
            "platform file close y",
            error=True,
            substrs=["'y' is not a valid file descriptor."],
        )
        self.expect(
            "platform file close -1",
            error=True,
            substrs=["'-1' is not a valid file descriptor."],
        )



"""
Test mpx-table command.
"""

import os
import time
import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMPXTable(TestBase):
    def setUp(self):
        TestBase.setUp(self)

    @skipIf(compiler="clang")
    @skipIf(oslist=no_match(["linux"]))
    @skipIf(archs=no_match(["i386", "x86_64"]))
    @skipIf(compiler="gcc", compiler_version=["<", "5"])  # GCC version >= 5 supports
    # Intel(R) Memory Protection Extensions (Intel(R) MPX).
    def test_show_command(self):
        """Test 'mpx-table show' command"""
        self.build()

        plugin_file = os.path.join(
            configuration.lldb_libs_dir, "liblldbIntelFeatures.so"
        )
        if not os.path.isfile(plugin_file):
            self.skipTest("features plugin missing.")
        plugin_command = " "
        seq = ("plugin", "load", plugin_file)
        plugin_command = plugin_command.join(seq)
        self.runCmd(plugin_command)

        exe = os.path.join(os.getcwd(), "a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.b1 = line_number("main.cpp", "// Break 1.")
        self.b2 = line_number("main.cpp", "// Break 2.")
        self.b3 = line_number("main.cpp", "// Break 3.")
        self.b4 = line_number("main.cpp", "// Break 4.")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.b1, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.b2, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.b3, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.b4, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        if process.GetState() == lldb.eStateExited:
            self.skipTest("Intel(R) MPX is not supported.")
        else:
            self.expect(
                "thread backtrace",
                STOPPED_DUE_TO_BREAKPOINT,
                substrs=["stop reason = breakpoint 1."],
            )

        self.expect(
            "mpx-table show a",
            substrs=[
                "lbound = 0x",
                ", ubound = 0x",
                "(pointer value = 0x",
                ", metadata = 0x",
                ")",
            ],
            error=False,
        )

        self.expect(
            "continue",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 2."],
        )

        # Check that out of scope pointer cannot be reached.
        #
        self.expect("mpx-table show a", substrs=["Invalid pointer."], error=True)

        self.expect(
            "mpx-table show tmp",
            substrs=[
                "lbound = 0x",
                ", ubound = 0x",
                "(pointer value = 0x",
                ", metadata = 0x",
                ")",
            ],
            error=False,
        )

        self.expect(
            "continue",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 3."],
        )

        # Check that the pointer value is correctly updated.
        #
        self.expect(
            "mpx-table show tmp",
            substrs=[
                "lbound = 0x",
                ", ubound = 0x",
                "(pointer value = 0x2",
                ", metadata = 0x",
                ")",
            ],
            error=False,
        )

        self.expect(
            "continue",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 4."],
        )

        # After going back to main(), check that out of scope pointer cannot be
        # reached.
        #
        self.expect("mpx-table show tmp", substrs=["Invalid pointer."], error=True)

        self.expect(
            "mpx-table show a",
            substrs=[
                "lbound = 0x",
                ", ubound = 0x",
                "(pointer value = 0x",
                ", metadata = 0x",
                ")",
            ],
            error=False,
        )

    def test_set_command(self):
        """Test 'mpx-table set' command"""
        self.build()

        plugin_file = os.path.join(
            configuration.lldb_libs_dir, "liblldbIntelFeatures.so"
        )
        if not os.path.isfile(plugin_file):
            self.skipTest("features plugin missing.")
        plugin_command = " "
        seq = ("plugin", "load", plugin_file)
        plugin_command = plugin_command.join(seq)
        self.runCmd(plugin_command)

        exe = os.path.join(os.getcwd(), "a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.b1 = line_number("main.cpp", "// Break 1.")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.b1, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        if process.GetState() == lldb.eStateExited:
            self.skipTest("Intel(R) MPX is not supported.")
        else:
            self.expect(
                "thread backtrace",
                STOPPED_DUE_TO_BREAKPOINT,
                substrs=["stop reason = breakpoint 1."],
            )

        # Check that the BT Entry doesn't already contain the test values.
        #
        self.expect(
            "mpx-table show a",
            matching=False,
            substrs=["lbound = 0xcafecafe", ", ubound = 0xbeefbeef"],
        )

        # Set the test values.
        #
        self.expect("mpx-table set a 0xcafecafe 0xbeefbeef", error=False)

        # Verify that the test values have been correctly written in the BT
        # entry.
        #
        self.expect(
            "mpx-table show a",
            substrs=["lbound = 0xcafecafe", ", ubound = 0xbeefbeef"],
            error=False,
        )



# This test checks that we make the executable the first
# element in the image list.

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestExecutableIsFirst(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # ELF does not have a hard distinction between shared libraries and
    # (position-independent) executables
    @skipIf(oslist=no_match(lldbplatformutil.getDarwinOSTriples() + ["windows"]))
    def test_executable_is_first_before_run(self):
        self.build()

        lib_name = self.platformContext.getFullLibName("bar")

        exe = self.getBuildArtifact("a.out")
        lib = self.getBuildArtifact(lib_name)

        target = self.dbg.CreateTarget(None)
        module = target.AddModule(lib, None, None)
        self.assertTrue(module.IsValid(), "Added the module for the library")

        module = target.AddModule(exe, None, None)
        self.assertTrue(module.IsValid(), "Added the executable module")

        # This is the executable module so it should be the first in the list:
        first_module = target.GetModuleAtIndex(0)
        print("This is the first test, this one succeeds")
        self.assertEqual(module, first_module, "This executable is the first module")

        # The executable property is an SBFileSpec to the executable.  Make sure
        # that is also right:
        executable_module = target.executable
        self.assertEqual(
            first_module.file, executable_module, "Python property is also our module"
        )

    def test_executable_is_first_during_run(self):
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self,
            "break after function call",
            lldb.SBFileSpec("main.cpp"),
            extra_images=["bar"],
        )

        first_module = target.GetModuleAtIndex(0)
        self.assertTrue(first_module.IsValid(), "We have at least one module")
        executable_module = target.executable
        self.assertEqual(first_module.file, executable_module, "They are the same")



"""
Test Unified Section List merging.
"""

import os
import shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbutil import symbol_type_to_str


class ModuleUnifiedSectionList(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipUnlessPlatform(["linux", "freebsd", "netbsd"])
    def test_unified_section_list(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        debug_info = self.getBuildArtifact("a.out.debug")
        new_dir = os.path.join(os.path.dirname(debug_info), "new_dir")
        os.mkdir(new_dir)
        renamed_debug_info = os.path.join(new_dir, "renamed.debug")
        os.rename(debug_info, renamed_debug_info)
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.assertGreater(target.GetNumModules(), 0)

        main_exe_module = target.GetModuleAtIndex(0)
        eh_frame = main_exe_module.FindSection(".eh_frame")
        self.assertTrue(eh_frame.IsValid())
        self.assertGreater(eh_frame.size, 0)

        # Should be stripped in main executable.
        debug_info_section = main_exe_module.FindSection(".debug_info")
        self.assertFalse(debug_info_section.IsValid())

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand(f"target symbols add {renamed_debug_info}", res)
        self.assertTrue(res.Succeeded())

        # Should be stripped in .debuginfo but be present in main executable.
        main_exe_module = target.GetModuleAtIndex(0)
        eh_frame = main_exe_module.FindSection(".eh_frame")
        self.assertTrue(eh_frame.IsValid())
        self.assertGreater(eh_frame.size, 0)

        # Should be unified and both sections should have contents.
        debug_info_section = main_exe_module.FindSection(".debug_info")
        self.assertTrue(debug_info_section.IsValid())
        self.assertGreater(debug_info_section.file_size, 0)

    def test_unified_section_list_overwrite_larger_section(self):
        """
        Test the merging of an ELF file with another ELF File where all the new sections are bigger, validating we
        overwrite .comment from SHT_NOBITS to the new SHT_PROGBITS section and the smaller .text with the larger
        .text
        """
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj("main.yaml", exe)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        main_exe_module = target.GetModuleAtIndex(0)

        # First we verify out .text section is the expected BEC0FFEE
        text_before_merge = main_exe_module.FindSection(".text")
        self.assertTrue(text_before_merge.IsValid())
        error = lldb.SBError()
        section_content = text_before_merge.data.ReadRawData(
            error, 0, text_before_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content, bytes.fromhex("BEC0FFEE"))

        # .comment in main.yaml should be SHT_NOBITS, and size 0
        comment_before_merge = main_exe_module.FindSection(".comment")
        self.assertTrue(comment_before_merge.IsValid())
        self.assertEqual(comment_before_merge.data.size, 0)

        # yamlize the main.largertext.yaml and force symbol loading
        debug_info = self.getBuildArtifact("a.out.debug")
        self.yaml2obj("main.largertext.yaml", debug_info)

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand(f"target symbols add {debug_info}", res)
        self.assertTrue(res.Succeeded())

        # verify we took the larger .text section
        main_exe_module_after_merge = target.GetModuleAtIndex(0)
        text_after_merge = main_exe_module_after_merge.FindSection(".text")
        self.assertTrue(text_after_merge.IsValid())
        self.assertGreater(text_after_merge.data.size, text_before_merge.data.size)
        section_content_after_merge = text_after_merge.data.ReadRawData(
            error, 0, text_after_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content_after_merge, bytes.fromhex("BEC0FFEEEEFF0CEB"))

        # in main.largertext.yaml comment is not SHT_NOBITS, and so we should see
        # the size > 0 and equal to BAADF00D
        comment_after_merge = main_exe_module_after_merge.FindSection(".comment")
        self.assertTrue(comment_after_merge.IsValid())
        comment_content_after_merge = comment_after_merge.data.ReadRawData(
            error, 0, comment_after_merge.data.size
        )

        self.assertTrue(error.Success())
        self.assertEqual(comment_content_after_merge, bytes.fromhex("BAADF00D"))

    def test_unified_section_list_overwrite_smaller_section(self):
        """
        Test the merging of an ELF file with another ELF File where all the existing sections are bigger, validating we don't
        overwrite with the SHT_NOBITS for .comment or the smaller .text section.
        """
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj("main.largertext.yaml", exe)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        main_exe_module = target.GetModuleAtIndex(0)

        # Same as above test but inverse, verify our larger .text section
        # is the expected BEC0FFEE palindrome
        text_before_merge = main_exe_module.FindSection(".text")
        self.assertTrue(text_before_merge.IsValid())
        error = lldb.SBError()
        section_content = text_before_merge.data.ReadRawData(
            error, 0, text_before_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content, bytes.fromhex("BEC0FFEEEEFF0CEB"))

        # Comment is SHT_PROGBITS on the larger yaml and should remain
        # the same after merge.
        comment_before_merge = main_exe_module.FindSection(".comment")
        self.assertTrue(comment_before_merge.IsValid())
        comment_content = comment_before_merge.data.ReadRawData(
            error, 0, comment_before_merge.data.size
        )

        self.assertTrue(error.Success())
        self.assertEqual(comment_content, bytes.fromhex("BAADF00D"))

        debug_info = self.getBuildArtifact("a.out.debug")
        self.yaml2obj("main.yaml", debug_info)

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand(f"target symbols add {debug_info}", res)
        self.assertTrue(res.Succeeded())

        # Verify we didn't replace the sections after merge.s
        main_exe_module_after_merge = target.GetModuleAtIndex(0)
        text_after_merge = main_exe_module_after_merge.FindSection(".text")
        self.assertTrue(text_after_merge.IsValid())
        self.assertEqual(text_after_merge.data.size, text_before_merge.data.size)
        section_content_after_merge = text_after_merge.data.ReadRawData(
            error, 0, text_after_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content_after_merge, bytes.fromhex("BEC0FFEEEEFF0CEB"))

        comment_after_merge = main_exe_module_after_merge.FindSection(".comment")
        self.assertTrue(comment_after_merge.IsValid())
        comment_content_after_merge = comment_after_merge.data.ReadRawData(
            error, 0, comment_after_merge.data.size
        )

        self.assertTrue(error.Success())
        self.assertEqual(comment_content_after_merge, bytes.fromhex("BAADF00D"))

    def test_unified_section_list_overwrite_mixed_merge(self):
        """
        Test the merging of an ELF file with another ELF File where the lhs has a larger .comment section
        and the RHS has a larger .text section.
        """
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj("main.largercomment.yaml", exe)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        main_exe_module = target.GetModuleAtIndex(0)

        # Verify we have the expected smaller BEC0FFEE
        text_before_merge = main_exe_module.FindSection(".text")
        self.assertTrue(text_before_merge.IsValid())
        error = lldb.SBError()
        section_content = text_before_merge.data.ReadRawData(
            error, 0, text_before_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content, bytes.fromhex("BEC0FFEE"))

        # Verify we have the larger palindromic comment
        comment_before_merge = main_exe_module.FindSection(".comment")
        self.assertTrue(comment_before_merge.IsValid())
        comment_content = comment_before_merge.data.ReadRawData(
            error, 0, comment_before_merge.data.size
        )

        self.assertTrue(error.Success())
        self.assertEqual(comment_content, bytes.fromhex("BAADF00DF00DBAAD"))

        debug_info = self.getBuildArtifact("a.out.debug")
        self.yaml2obj("main.largertext.yaml", debug_info)

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand(f"target symbols add {debug_info}", res)
        self.assertTrue(res.Succeeded())

        # Verify we replaced .text
        main_exe_module_after_merge = target.GetModuleAtIndex(0)
        text_after_merge = main_exe_module_after_merge.FindSection(".text")
        self.assertTrue(text_after_merge.IsValid())
        section_content_after_merge = text_after_merge.data.ReadRawData(
            error, 0, text_after_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content_after_merge, bytes.fromhex("BEC0FFEEEEFF0CEB"))

        # Verify .comment is still the same.
        comment_after_merge = main_exe_module_after_merge.FindSection(".comment")
        self.assertTrue(comment_after_merge.IsValid())
        comment_content_after_merge = comment_after_merge.data.ReadRawData(
            error, 0, comment_after_merge.data.size
        )

        self.assertTrue(error.Success())
        self.assertEqual(comment_content_after_merge, bytes.fromhex("BAADF00DF00DBAAD"))

    def test_unified_section_list_overwrite_equal_size(self):
        """
        Test the merging of an ELF file with an ELF file with sections of the same size with different values
        .text
        """
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj("main.yaml", exe)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        main_exe_module = target.GetModuleAtIndex(0)

        # First we verify out .text section is the expected BEC0FFEE
        text_before_merge = main_exe_module.FindSection(".text")
        self.assertTrue(text_before_merge.IsValid())
        error = lldb.SBError()
        section_content = text_before_merge.data.ReadRawData(
            error, 0, text_before_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content, bytes.fromhex("BEC0FFEE"))

        # .comment in main.yaml should be SHT_NOBITS, and size 0
        comment_before_merge = main_exe_module.FindSection(".comment")
        self.assertTrue(comment_before_merge.IsValid())
        self.assertEqual(comment_before_merge.data.size, 0)

        # yamlize the main with the .text reversed from BEC0FFEE
        # to EEFF0CEB. We should still keep our .text with BEC0FFEE
        debug_info = self.getBuildArtifact("a.out.debug")
        self.yaml2obj("main.reversedtext.yaml", debug_info)

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand(f"target symbols add {debug_info}", res)
        self.assertTrue(res.Succeeded())

        # verify .text did not change
        main_exe_module_after_merge = target.GetModuleAtIndex(0)
        text_after_merge = main_exe_module_after_merge.FindSection(".text")
        self.assertTrue(text_after_merge.IsValid())
        section_content_after_merge = text_after_merge.data.ReadRawData(
            error, 0, text_after_merge.data.size
        )
        self.assertTrue(error.Success())
        self.assertEqual(section_content_after_merge, bytes.fromhex("BEC0FFEE"))

        # verify comment did not change
        comment_afer_merge = main_exe_module_after_merge.FindSection(".comment")
        self.assertTrue(comment_afer_merge.IsValid())
        self.assertEqual(comment_afer_merge.data.size, 0)



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbpexpect import PExpectTest
from lldbsuite.test.lldbtest import *


class TestCase(PExpectTest):
    def expect_repl(self, expr, substrs=[]):
        """Evaluates the expression in the REPL and verifies that the list
        of substrs is in the REPL output."""
        # Only single line expressions supported.
        self.assertNotIn("\n", expr)
        self.child.send(expr + "\n")
        for substr in substrs:
            self.child.expect_exact(substr)
        # Look for the start of the next REPL input line.
        self.current_repl_line_number += 1
        self.child.expect_exact(str(self.current_repl_line_number) + ">")

    def start_repl(self):
        self.build()
        self.current_repl_line_number = 1

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        # Try launching the REPL before we have a running target.
        self.expect(
            "expression --repl -l c --",
            substrs=["REPL requires a running target process."],
        )

        self.expect("b main", substrs=["Breakpoint 1", "address ="])
        self.expect("run", substrs=["stop reason = breakpoint 1"])

        # Start the REPL.
        self.child.send("expression --repl -l c --\n")
        self.child.expect_exact("1>")

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])  # Randomly fails on buildbot
    @skipIfEditlineSupportMissing
    def test_basic_completion(self):
        """Test that we can complete a simple multiline expression"""
        self.start_repl()
        # Try evaluating a simple expression.
        self.expect_repl("3 + 3", substrs=["(int) $0 = 6"])

        # Try declaring a persistent variable.
        self.expect_repl(
            "long $persistent = 7; 5",
            substrs=["(int) $1 = 5", "(long) $persistent = 7"],
        )

        # Try using the persistent variable from before.
        self.expect_repl("$persistent + 10", substrs=["(long) $2 = 17"])

        self.quit()

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])  # Randomly fails on buildbot
    @skipIfEditlineSupportMissing
    def test_completion_with_space_only_line(self):
        """Test that we don't crash when completing lines with spaces only"""
        self.start_repl()

        self.child.send("   ")
        self.child.send("\t")
        self.expect_repl("3 + 3", substrs=["(int) $0 = 6"])



"""
Test that recursive types are handled correctly.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class RecursiveTypesTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.
        self.line = line_number("recursive_type_main.cpp", "// Test at this line.")

        self.d1 = {"CXX_SOURCES": "recursive_type_main.cpp recursive_type_1.cpp"}
        self.d2 = {"CXX_SOURCES": "recursive_type_main.cpp recursive_type_2.cpp"}

    def test_recursive_type_1(self):
        """Test that recursive structs are displayed correctly."""
        self.build(dictionary=self.d1)
        self.setTearDownCleanup(dictionary=self.d1)
        self.print_struct()

    def test_recursive_type_2(self):
        """Test that recursive structs are displayed correctly."""
        self.build(dictionary=self.d2)
        self.setTearDownCleanup(dictionary=self.d2)
        self.print_struct()

    def print_struct(self):
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "recursive_type_main.cpp",
            self.line,
            num_expected_locations=-1,
            loc_exact=True,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        self.runCmd("expression tpi")
        self.runCmd("expression *tpi")



"""
Test the "process continue -b" option.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestContinueToBkpts(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["pyapi"])
    def test_continue_to_breakpoints(self):
        """Test that the continue to breakpoints feature works correctly."""
        self.build()
        self.do_test_continue_to_breakpoint()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.main_source_spec = lldb.SBFileSpec("main.c")

    def continue_and_check(self, stop_list, bkpt_to_hit, loc_to_hit=0):
        """Build up a command that will run a continue -b commands using the breakpoints on stop_list, and
        ensure that we hit bkpt_to_hit.
        If loc_to_hit is not 0, also verify that we hit that location."""
        command = "process continue"
        for elem in stop_list:
            command += " -b {0}".format(elem)
        self.expect(command)
        self.assertStopReason(
            self.thread.stop_reason, lldb.eStopReasonBreakpoint, "Hit a breakpoint"
        )
        self.assertEqual(
            self.thread.GetStopReasonDataAtIndex(0),
            bkpt_to_hit,
            "Hit the right breakpoint",
        )
        if loc_to_hit != 0:
            self.assertEqual(
                self.thread.GetStopReasonDataAtIndex(1),
                loc_to_hit,
                "Hit the right location",
            )
        for bkpt_id in self.bkpt_list:
            bkpt = self.target.FindBreakpointByID(bkpt_id)
            self.assertTrue(bkpt.IsValid(), "Breakpoint id's round trip")
            if bkpt.MatchesName("disabled"):
                self.assertFalse(
                    bkpt.IsEnabled(),
                    "Disabled breakpoints stay disabled: {0}".format(bkpt.GetID()),
                )
            else:
                self.assertTrue(
                    bkpt.IsEnabled(),
                    "Enabled breakpoints stay enabled: {0}".format(bkpt.GetID()),
                )
        # Also do our multiple location one:
        bkpt = self.target.FindBreakpointByID(self.multiple_loc_id)
        self.assertTrue(bkpt.IsValid(), "Breakpoint with locations round trip")
        for i in range(1, 3):
            loc = bkpt.FindLocationByID(i)
            self.assertTrue(loc.IsValid(), "Locations round trip")
            if i == 2:
                self.assertTrue(
                    loc.IsEnabled(), "Locations that were enabled stay enabled"
                )
            else:
                self.assertFalse(
                    loc.IsEnabled(), "Locations that were disabled stay disabled"
                )

    def do_test_continue_to_breakpoint(self):
        """Test the continue to breakpoint feature."""
        (self.target, process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here to get started", self.main_source_spec
        )

        # Now set up all our breakpoints:
        bkpt_pattern = "This is the {0} stop"
        bkpt_elements = [
            "zeroth",
            "first",
            "second",
            "third",
            "fourth",
            "fifth",
            "sixth",
            "seventh",
            "eighth",
            "nineth",
        ]
        disabled_bkpts = ["first", "eigth"]
        bkpts_for_MyBKPT = ["first", "sixth", "nineth"]
        self.bkpt_list = []
        for elem in bkpt_elements:
            bkpt = self.target.BreakpointCreateBySourceRegex(
                bkpt_pattern.format(elem), self.main_source_spec
            )
            self.assertGreater(bkpt.GetNumLocations(), 0, "Found a bkpt match")
            self.bkpt_list.append(bkpt.GetID())
            bkpt.AddName(elem)
            if elem in disabled_bkpts:
                bkpt.AddName("disabled")
                bkpt.SetEnabled(False)
            if elem in bkpts_for_MyBKPT:
                bkpt.AddName("MyBKPT")
        # Also make one that has several locations, so we can test locations:
        mult_bkpt = self.target.BreakpointCreateBySourceRegex(
            bkpt_pattern.format("(seventh|eighth|nineth)"), self.main_source_spec
        )
        self.assertEqual(mult_bkpt.GetNumLocations(), 3, "Got three matches")
        mult_bkpt.AddName("Locations")
        # Disable all of these:
        for i in range(1, 4):
            loc = mult_bkpt.FindLocationByID(i)
            self.assertTrue(loc.IsValid(), "Location {0} is valid".format(i))
            loc.SetEnabled(False)
            self.assertFalse(loc.IsEnabled(), "Loc {0} wasn't disabled".format(i))
        self.multiple_loc_id = mult_bkpt.GetID()

        # First test out various error conditions

        # All locations of the multiple_loc_id are disabled, so running to this should be an error:
        self.expect(
            "process continue -b {0}".format(self.multiple_loc_id),
            error=True,
            msg="Running to a disabled breakpoint by number",
        )

        # Now re-enable the middle one so we can run to it:
        loc = mult_bkpt.FindLocationByID(2)
        loc.SetEnabled(True)

        self.expect(
            "process continue -b {0}".format(self.bkpt_list[1]),
            error=True,
            msg="Running to a disabled breakpoint by number",
        )
        self.expect(
            "process continue -b {0}.1".format(self.bkpt_list[1]),
            error=True,
            msg="Running to a location of a disabled breakpoint",
        )
        self.expect(
            "process continue -b disabled",
            error=True,
            msg="Running to a disabled set of breakpoints",
        )
        self.expect(
            "process continue -b {0}.{1}".format(self.multiple_loc_id, 1),
            error=True,
            msg="Running to a disabled breakpoint location",
        )
        self.expect(
            "process continue -b {0}".format("THERE_ARE_NO_BREAKPOINTS_BY_THIS_NAME"),
            error=True,
            msg="Running to no such name",
        )
        self.expect(
            "process continue -b {0}".format(1000),
            error=True,
            msg="Running to no such breakpoint",
        )
        self.expect(
            "process continue -b {0}.{1}".format(self.multiple_loc_id, 1000),
            error=True,
            msg="Running to no such location",
        )

        # Now move forward, this time with breakpoint numbers.  First time we don't skip other bkpts.
        bkpt = self.bkpt_list[0]
        self.continue_and_check([str(bkpt)], bkpt)

        # Now skip to the third stop, do it by name and supply one of the later breakpoints as well:
        # This continue has to muck with the sync mode of the debugger, so let's make sure we
        # put it back.  First try if it was in sync mode:
        orig_async = self.dbg.GetAsync()
        self.dbg.SetAsync(True)
        self.continue_and_check([bkpt_elements[2], bkpt_elements[7]], self.bkpt_list[2])
        after_value = self.dbg.GetAsync()
        self.dbg.SetAsync(orig_async)
        self.assertTrue(after_value, "Preserve async as True if it started that way")

        # Now try a name that has several breakpoints.
        # This time I'm also going to check that we put the debugger async mode back if
        # if was False to begin with:
        self.dbg.SetAsync(False)
        self.continue_and_check(["MyBKPT"], self.bkpt_list[6])
        after_value = self.dbg.GetAsync()
        self.dbg.SetAsync(orig_async)
        self.assertFalse(after_value, "Preserve async as False if it started that way")

        # Now let's run to a particular location.  Also specify a breakpoint we've already hit:
        self.continue_and_check(
            [self.bkpt_list[0], self.multiple_loc_id], self.multiple_loc_id, 2
        )



"""
Test that we can backtrace correctly when AArch64 PAC is enabled
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64UnwindPAC(TestBase):
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test(self):
        """Test that we can backtrace correctly when AArch64 PAC is enabled"""
        if not self.isAArch64PAuth():
            self.skipTest("Target must support Pointer Authentication.")

        self.build()

        self.line = line_number("main.c", "// Frame func_c")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()
        thread = process.GetThreadAtIndex(0)

        backtrace = [
            "func_c",
            "func_b",
            "func_a",
            "main",
        ]

        libc_backtrace = [
            "__libc_start_main",
            "_start",
        ]

        self.assertGreaterEqual(
            thread.GetNumFrames(), len(backtrace) + len(libc_backtrace)
        )

        # Strictly check frames that are in the test program's source.
        for frame_idx, frame in enumerate(thread.frames[: len(backtrace)]):
            self.assertTrue(frame)
            self.assertEqual(frame.GetFunctionName(), backtrace[frame_idx])
            self.assertEqual(
                frame.GetLineEntry().GetLine(),
                line_number("main.c", "Frame " + backtrace[frame_idx]),
            )

        # After the program comes some libc frames. The number varies by
        # system, so ensure we have at least these two in this order,
        # skipping frames in between.
        start_idx = frame_idx + 1
        for frame_idx, frame in enumerate(thread.frames[start_idx:], start=start_idx):
            self.assertTrue(frame)
            if libc_backtrace[0] == frame.GetFunctionName():
                libc_backtrace.pop(0)

        self.assertFalse(libc_backtrace, "Did not find expected libc frames.")



"""Test that adding, deleting and modifying watchpoints sends the appropriate events."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestWatchpointEvents(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"

    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test that adding, deleting and modifying watchpoints sends the appropriate events."""
        self.build()
        target = self.createTestTarget()

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        break_in_main = target.BreakpointCreateBySourceRegex(
            "// Put a breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break_in_main, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_in_main)

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        thread = threads[0]
        frame = thread.GetFrameAtIndex(0)
        local_var = frame.FindVariable("local_var")
        self.assertTrue(local_var.IsValid())

        self.listener = lldb.SBListener("com.lldb.testsuite_listener")
        self.target_bcast = target.GetBroadcaster()
        self.target_bcast.AddListener(
            self.listener, lldb.SBTarget.eBroadcastBitWatchpointChanged
        )
        self.listener.StartListeningForEvents(
            self.target_bcast, lldb.SBTarget.eBroadcastBitWatchpointChanged
        )

        error = lldb.SBError()
        local_watch = local_var.Watch(True, False, True, error)
        if not error.Success():
            self.fail(
                "Failed to make watchpoint for local_var: %s" % (error.GetCString())
            )

        self.GetWatchpointEvent(lldb.eWatchpointEventTypeAdded)
        # Now change some of the features of this watchpoint and make sure we
        # get events:
        local_watch.SetEnabled(False)
        self.GetWatchpointEvent(lldb.eWatchpointEventTypeDisabled)

        local_watch.SetEnabled(True)
        self.GetWatchpointEvent(lldb.eWatchpointEventTypeEnabled)

        local_watch.SetIgnoreCount(10)
        self.GetWatchpointEvent(lldb.eWatchpointEventTypeIgnoreChanged)

        condition = "1 == 2"
        local_watch.SetCondition(condition)
        self.GetWatchpointEvent(lldb.eWatchpointEventTypeConditionChanged)

        self.assertEqual(
            local_watch.GetCondition(),
            condition,
            'make sure watchpoint condition is "' + condition + '"',
        )

        target.DeleteWatchpoint(local_watch.GetID())
        self.GetWatchpointEvent(
            lldb.eWatchpointEventTypeDisabled, lldb.eWatchpointEventTypeRemoved
        )

        # Re-create it so that we can check DeleteAllWatchpoints
        local_watch = local_var.Watch(True, False, True, error)
        if not error.Success():
            self.fail(
                "Failed to make watchpoint for local_var: %s" % (error.GetCString())
            )
        self.GetWatchpointEvent(lldb.eWatchpointEventTypeAdded)
        target.DeleteAllWatchpoints()
        self.GetWatchpointEvent(
            lldb.eWatchpointEventTypeDisabled, lldb.eWatchpointEventTypeRemoved
        )

    def GetWatchpointEvent(self, *event_types):
        # We added a watchpoint so we should get a watchpoint added event.
        event = lldb.SBEvent()
        for event_type in event_types:
            success = self.listener.WaitForEvent(1, event)
            self.assertTrue(success, "Successfully got watchpoint event")
            self.assertTrue(
                lldb.SBWatchpoint.EventIsWatchpointEvent(event),
                "Event is a watchpoint event.",
            )
            found_type = lldb.SBWatchpoint.GetWatchpointEventTypeFromEvent(event)
            self.assertEqual(
                found_type,
                event_type,
                "Event is not correct type, expected: %d, found: %d"
                % (event_type, found_type),
            )
        # There shouldn't be another event waiting around:
        found_event = self.listener.PeekAtNextEventForBroadcasterWithType(
            self.target_bcast, lldb.SBTarget.eBroadcastBitWatchpointChanged, event
        )
        if found_event:
            print("Found an event I didn't expect: ", event.GetType())

        self.assertTrue(not found_event, f"Only expected {len(event_types)} events.")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.builtin as builtin
import mlir.dialects.func as func
import mlir.dialects.x86 as x86


def run(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        f()
    return f


# CHECK-LABEL: TEST: testAvxOp
@run
def testAvxOp():
    module = Module.create()
    with InsertionPoint(module.body):

        @func.FuncOp.from_py_func(MemRefType.get((1,), BF16Type.get()))
        def avx_op(arg):
            return x86.BcstToPackedF32Op(a=arg, dst=VectorType.get((8,), F32Type.get()))

    # CHECK-LABEL: func @avx_op(
    # CHECK-SAME:      %[[ARG:.+]]: memref<1xbf16>) -> vector<8xf32> {
    #       CHECK:   %[[VAL:.+]] = x86.avx.bcst_to_f32.packed %[[ARG]]
    #       CHECK:   return %[[VAL]] : vector<8xf32>
    #       CHECK: }
    print(module)


# CHECK-LABEL: TEST: testAvx512Op
@run
def testAvx512Op():
    module = Module.create()
    with InsertionPoint(module.body):

        @func.FuncOp.from_py_func(VectorType.get((8,), F32Type.get()))
        def avx512_op(arg):
            return x86.CvtPackedF32ToBF16Op(
                a=arg, dst=VectorType.get((8,), BF16Type.get())
            )

    # CHECK-LABEL: func @avx512_op(
    # CHECK-SAME:      %[[ARG:.+]]: vector<8xf32>) -> vector<8xbf16> {
    #       CHECK:   %[[VAL:.+]] = x86.avx512.cvt.packed.f32_to_bf16 %[[ARG]]
    #       CHECK:   return %[[VAL]] : vector<8xbf16>
    #       CHECK: }
    print(module)


# CHECK-LABEL: TEST: testAvx10Op
@run
def testAvx10Op():
    module = Module.create()
    with InsertionPoint(module.body):

        @func.FuncOp.from_py_func(
            VectorType.get((16,), IntegerType.get(32)),
            VectorType.get((64,), IntegerType.get(8)),
            VectorType.get((64,), IntegerType.get(8)),
        )
        def avx10_op(*args):
            return x86.AVX10DotInt8Op(w=args[0], a=args[1], b=args[2])

    # CHECK-LABEL: func @avx10_op(
    # CHECK-SAME:      %[[W:.+]]: vector<16xi32>, %[[A:.+]]: vector<64xi8>,
    # CHECK-SAME:      %[[B:.+]]: vector<64xi8>) -> vector<16xi32> {
    #       CHECK:   %[[VAL:.+]] = x86.avx10.dot.i8 %[[W]], %[[A]], %[[B]]
    #       CHECK:   return %[[VAL]] : vector<16xi32>
    #       CHECK: }
    print(module)



"""This file contains functions for discovering benchmark functions. It works
in a similar way to python's unittest library.
"""
import configparser
import importlib
import os
import pathlib
import re
import sys
import types


def discover_benchmark_modules(top_level_path):
    """Starting from the `top_level_path`, discover python files which contains
    benchmark functions. It looks for files with a specific prefix, which
    defaults to "benchmark_"
    """
    config = configparser.ConfigParser()
    config.read(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.ini"))
    if "discovery" in config.sections():
        filename_prefix = config["discovery"]["filename_prefix"]
    else:
        filename_prefix = "benchmark_"
    if re.search(rf"{filename_prefix}.*.py$", top_level_path):
        # A specific python file so just include that.
        benchmark_files = [top_level_path]
    else:
        # A directory so recursively search for all python files.
        benchmark_files = pathlib.Path(top_level_path).rglob(f"{filename_prefix}*.py")
    for benchmark_filename in benchmark_files:
        benchmark_abs_dir = os.path.abspath(os.path.dirname(benchmark_filename))
        sys.path.append(benchmark_abs_dir)
        module_file_name = os.path.basename(benchmark_filename)
        module_name = module_file_name.replace(".py", "")
        module = importlib.import_module(module_name)
        yield module
        sys.path.pop()


def get_benchmark_functions(module, benchmark_function_name=None):
    """Discover benchmark functions in python file. It looks for functions with
    a specific prefix, which defaults to "benchmark_".
    """
    config = configparser.ConfigParser()
    config.read(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.ini"))
    if "discovery" in config.sections():
        function_prefix = config["discovery"].get("function_prefix")
    else:
        function_prefix = "benchmark_"

    module_functions = []
    for attribute_name in dir(module):
        attribute = getattr(module, attribute_name)
        if isinstance(attribute, types.FunctionType) and attribute_name.startswith(
            function_prefix
        ):
            module_functions.append(attribute)

    if benchmark_function_name:
        # If benchmark_function_name is present, just yield the corresponding
        # function and nothing else.
        for function in module_functions:
            if function.__name__ == benchmark_function_name:
                yield function
    else:
        # If benchmark_function_name is not present, yield all functions.
        for function in module_functions:
            yield function



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        self.expect_expr("$__lldb_expr_result", result_type="int", result_value="11")
        self.expect_expr("$foo", result_type="int", result_value="12")
        self.expect_expr("$R0", result_type="int", result_value="13")
        self.expect(
            "expr int $foo = 123", error=True, substrs=["declaration conflicts"]
        )
        self.expect_expr("$0", result_type="int", result_value="11")
